@@ -4,8 +4,6 @@
  *
  * Include all the necessary files to use the BinarySpaceTree class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -18,6 +16,10 @@
 #include "bounds.hpp"
 #include "binary_space_tree/midpoint_split.hpp"
 #include "binary_space_tree/mean_split.hpp"
+#include "binary_space_tree/vantage_point_split.hpp"
+#include "binary_space_tree/rp_tree_max_split.hpp"
+#include "binary_space_tree/rp_tree_mean_split.hpp"
+#include "binary_space_tree/ub_tree_split.hpp"
 #include "binary_space_tree/binary_space_tree.hpp"
 #include "binary_space_tree/single_tree_traverser.hpp"
 #include "binary_space_tree/single_tree_traverser_impl.hpp"
