@@ -7,8 +7,6 @@
  * tree node into two parts using the midpoint of the values in a certain
  * dimension.  The dimension to split on is the dimension with maximum variance.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -18,6 +16,7 @@
 #define MLPACK_CORE_TREE_BINARY_SPACE_TREE_MIDPOINT_SPLIT_HPP
 
 #include <mlpack/core.hpp>
+#include <mlpack/core/tree/perform_split.hpp>
 
 namespace mlpack {
 namespace tree /** Trees and tree-building procedures. */ {
@@ -31,88 +30,93 @@ template<typename BoundType, typename MatType = arma::mat>
 class MidpointSplit
 {
  public:
+  //! A struct that contains an information about the split.
+  struct SplitInfo
+  {
+    //! The dimension to split the node on.
+    size_t splitDimension;
+    //! The split in dimension splitDimension is based on this value.
+    double splitVal;
+  };
   /**
-   * Split the node according to the mean value in the dimension with maximum
-   * width.
+   * Find the partition of the node. This method fills up the dimension that
+   * will be used to split the node and the value according which the split
+   * will be performed.
    *
    * @param bound The bound used for this node.
    * @param data The dataset used by the binary space tree.
    * @param begin Index of the starting point in the dataset that belongs to
    *    this node.
    * @param count Number of points in this node.
-   * @param splitDimension This will be filled with the dimension the node is to
-   *    be split on.
-   * @param splitCol The index at which the dataset is divided into two parts
-   *    after the rearrangement.
+   * @param splitInfo An information about the split. This information contains
+   *    the dimension and the value.
    */
   static bool SplitNode(const BoundType& bound,
                         MatType& data,
                         const size_t begin,
                         const size_t count,
-                        size_t& splitCol);
+                        SplitInfo& splitInfo);
 
   /**
-   * Split the node according to the mean value in the dimension with maximum
-   * width and return a list of changed indices.
+   * Perform the split process according to the information about the
+   * split. This will order the dataset such that points that belong to the left
+   * subtree are on the left of the split column, and points from the right
+   * subtree are on the right side of the split column.
    *
    * @param bound The bound used for this node.
    * @param data The dataset used by the binary space tree.
    * @param begin Index of the starting point in the dataset that belongs to
    *    this node.
    * @param count Number of points in this node.
-   * @param splitDimension This will be filled with the dimension the node is
-   *    to be split on.
-   * @param splitCol The index at which the dataset is divided into two parts
-   *    after the rearrangement.
-   * @param oldFromNew Vector which will be filled with the old positions for
-   *    each new point.
-   */
-  static bool SplitNode(const BoundType& bound,
-                        MatType& data,
-                        const size_t begin,
-                        const size_t count,
-                        size_t& splitCol,
-                        std::vector<size_t>& oldFromNew);
-
- private:
-  /**
-   * Reorder the dataset into two parts such that they lie on either side of
-   * splitCol.
-   *
-   * @param data The dataset used by the binary space tree.
-   * @param begin Index of the starting point in the dataset that belongs to
-   *    this node.
-   * @param count Number of points in this node.
-   * @param splitDimension The dimension to split the node on.
-   * @param splitVal The split in dimension splitDimension is based on this
-   *    value.
+   * @param splitInfo The information about the split.
    */
   static size_t PerformSplit(MatType& data,
                              const size_t begin,
                              const size_t count,
-                             const size_t splitDimension,
-                             const double splitVal);
+                             const SplitInfo& splitInfo)
+  {
+    return split::PerformSplit<MatType, MidpointSplit>(data, begin, count,
+        splitInfo);
+  }
 
   /**
-   * Reorder the dataset into two parts such that they lie on either side of
-   * splitCol. Also returns a list of changed indices.
+   * Perform the split process according to the information about the split and
+   * return the list of changed indices. This will order the dataset such that
+   * points that belong to the left subtree are on the left of the split column,
+   * and points from the right subtree are on the right side of the split
+   * column.
    *
+   * @param bound The bound used for this node.
    * @param data The dataset used by the binary space tree.
    * @param begin Index of the starting point in the dataset that belongs to
    *    this node.
    * @param count Number of points in this node.
-   * @param splitDimension The dimension to split the node on.
-   * @param splitVal The split in dimension splitDimension is based on this
-   *    value.
+   * @param splitInfo The information about the split.
    * @param oldFromNew Vector which will be filled with the old positions for
    *    each new point.
    */
   static size_t PerformSplit(MatType& data,
                              const size_t begin,
                              const size_t count,
-                             const size_t splitDimension,
-                             const double splitVal,
-                             std::vector<size_t>& oldFromNew);
+                             const SplitInfo& splitInfo,
+                             std::vector<size_t>& oldFromNew)
+  {
+    return split::PerformSplit<MatType, MidpointSplit>(data, begin, count,
+        splitInfo, oldFromNew);
+  }
+
+  /**
+   * Indicates that a point should be assigned to the left subtree.
+   *
+   * @param point The point that is being assigned.
+   * @param splitInfo An information about the split.
+   */
+  template<typename VecType>
+  static bool AssignToLeftNode(const VecType& point,
+                               const SplitInfo& splitInfo)
+  {
+    return point[splitInfo.splitDimension] < splitInfo.splitVal;
+  }
 };
 
 } // namespace tree
