@@ -5,8 +5,6 @@
  * Definition of RStarTreeDescentHeuristic, a class that chooses the best child
  * of a node in an R tree when inserting a new point.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -21,25 +19,25 @@ namespace mlpack {
 namespace tree {
 
 /**
- * When descending a Rectangle tree to insert a point, we need to have a way to
+ * When descending a RectangleTree to insert a point, we need to have a way to
  * choose a child node when the point isn't enclosed by any of them.  This
- * heuristic is used to do so.
+ * heuristic is used to do so using the rules for the R* tree.
  */
 class RStarTreeDescentHeuristic
 {
  public:
   /**
-   * Evaluate the node using a hueristic.  The heuristic guarantees two things:
+   * Evaluate the node using a heuristic.  The heuristic guarantees two things:
    *
-   * 1.  If point is contained in (or on) bound, the value returned is zero.
-   * 2.  If the point is not contained in (or on) bound, the value returned is
-   *     greater than zero.
+   * 1. If point is contained in (or on) bound, the value returned is zero.
+   * 2. If the point is not contained in (or on) bound, the value returned is
+   *    greater than zero.
    *
    * @param bound The bound used for the node that is being evaluated.
-   * @param point The point that is being inserted.
+   * @param point The index of the point that is being inserted.
    */
   template<typename TreeType>
-  static size_t ChooseDescentNode(const TreeType* node, const arma::vec& point);
+  static size_t ChooseDescentNode(const TreeType* node, const size_t point);
 
   template<typename TreeType>
   static size_t ChooseDescentNode(const TreeType* node,
