@@ -2,11 +2,9 @@
  * @file r_tree_star_split.hpp
  * @author Andrew Wells
  *
- * Defintion of the RStarTreeSplit class, a class that splits the nodes of an R tree, starting
+ * Definition of the RStarTreeSplit class, a class that splits the nodes of an R tree, starting
  * at a leaf node and moving upwards if necessary.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -25,45 +23,31 @@ namespace tree /** Trees and tree-building procedures. */ {
  * nodes overflow, we split them, moving up the tree and splitting nodes
  * as necessary.
  */
-template <typename TreeType>
 class RStarTreeSplit
 {
  public:
-  //! Default constructor.
-  RStarTreeSplit() { }
-
-  //! Construct this with the specified node.
-  RStarTreeSplit(const TreeType* /* node */) { }
-
-  //! Create a copy of the other.split.
-  RStarTreeSplit(const TreeType& /* other */) { }
-
   /**
    * Split a leaf node using the algorithm described in "The R*-tree: An
    * Efficient and Robust Access method for Points and Rectangles."  If
    * necessary, this split will propagate upwards through the tree.
    */
-  void SplitLeafNode(TreeType* tree, std::vector<bool>& relevels);
+  template <typename TreeType>
+  static void SplitLeafNode(TreeType *tree,std::vector<bool>& relevels);
 
   /**
    * Split a non-leaf node using the "default" algorithm.  If this is a root
    * node, the tree increases in depth.
    */
-  bool SplitNonLeafNode(TreeType* tree, std::vector<bool>& relevels);
+  template <typename TreeType>
+  static bool SplitNonLeafNode(TreeType *tree,std::vector<bool>& relevels);
 
  private:
   /**
    * Insert a node into another node.
    */
+  template <typename TreeType>
   static void InsertNodeIntoTree(TreeType* destTree, TreeType* srcNode);
 
- public:
-  /**
-   * Serialize the split.
-   */
-  template<typename Archive>
-  void Serialize(Archive &, const unsigned int /* version */) { };
-
   /**
    * Comparator for sorting with std::pair. This comparator works a little bit
    * faster then the default comparator.
