@@ -2,11 +2,9 @@
  * @file r_tree_split.hpp
  * @author Andrew Wells
  *
- * Defintion of the RTreeSplit class, a class that splits the nodes of an R
+ * Definition of the RTreeSplit class, a class that splits the nodes of an R
  * tree, starting at a leaf node and moving upwards if necessary.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -25,45 +23,40 @@ namespace tree /** Trees and tree-building procedures. */ {
  * nodes overflow, we split them, moving up the tree and splitting nodes
  * as necessary.
  */
-template<typename TreeType>
 class RTreeSplit
 {
  public:
-  //! Default constructor.
-  RTreeSplit() { }
-
-  //! Construct this with the specified node.
-  RTreeSplit(const TreeType* /* node */) { }
-
-  //! Create a copy of the other split.
-  RTreeSplit(const TreeType& /* other */) { }
-
   /**
    * Split a leaf node using the "default" algorithm.  If necessary, this split
    * will propagate upwards through the tree.
    */
-  void SplitLeafNode(TreeType* tree, std::vector<bool>& relevels);
+  template<typename TreeType>
+  static void SplitLeafNode(TreeType *tree,std::vector<bool>& relevels);
 
   /**
    * Split a non-leaf node using the "default" algorithm.  If this is a root
    * node, the tree increases in depth.
    */
-  bool SplitNonLeafNode(TreeType* tree, std::vector<bool>& relevels);
+  template<typename TreeType>
+  static bool SplitNonLeafNode(TreeType *tree,std::vector<bool>& relevels);
 
  private:
   /**
    * Get the seeds for splitting a leaf node.
    */
-  static void GetPointSeeds(const TreeType* tree, int& i, int& j);
+  template<typename TreeType>
+  static void GetPointSeeds(const TreeType *tree,int& i, int& j);
 
   /**
    * Get the seeds for splitting a non-leaf node.
    */
-  static void GetBoundSeeds(const TreeType* tree, int& i, int& j);
+  template<typename TreeType>
+  static void GetBoundSeeds(const TreeType *tree,int& i, int& j);
 
   /**
    * Assign points to the two new nodes.
    */
+  template<typename TreeType>
   static void AssignPointDestNode(TreeType* oldTree,
                                   TreeType* treeOne,
                                   TreeType* treeTwo,
@@ -73,6 +66,7 @@ class RTreeSplit
   /**
    * Assign nodes to the two new nodes.
    */
+  template<typename TreeType>
   static void AssignNodeDestNode(TreeType* oldTree,
                                  TreeType* treeOne,
                                  TreeType* treeTwo,
@@ -82,15 +76,8 @@ class RTreeSplit
   /**
    * Insert a node into another node.
    */
+  template<typename TreeType>
   static void InsertNodeIntoTree(TreeType* destTree, TreeType* srcNode);
-
- public:
-  /**
-   * Serialize the split.
-   */
-  template<typename Archive>
-  void Serialize(Archive &, const unsigned int /* version */) { };
-
 };
 
 } // namespace tree
