@@ -4,8 +4,6 @@
  *
  * Implementation of class (XTreeSplit) to split a RectangleTree.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -21,32 +19,6 @@
 namespace mlpack {
 namespace tree {
 
-template<typename TreeType>
-XTreeSplit<TreeType>::XTreeSplit() :
-    normalNodeMaxNumChildren(0),
-    splitHistory(0)
-{
-
-}
-
-template<typename TreeType>
-XTreeSplit<TreeType>::XTreeSplit(const TreeType*node) :
-    normalNodeMaxNumChildren(node->Parent() ?
-        node->Parent()->Split().NormalNodeMaxNumChildren() :
-        node->MaxNumChildren()),
-    splitHistory(node->Bound().Dim())
-{
-
-}
-
-template<typename TreeType>
-XTreeSplit<TreeType>::XTreeSplit(const TreeType &other) :
-    normalNodeMaxNumChildren(other.Split().NormalNodeMaxNumChildren()),
-    splitHistory(other.Split().SplitHistory())
-{
-
-}
-
 /**
  * We call GetPointSeeds to get the two points which will be the initial points
  * in the new nodes We then call AssignPointDestNode to assign the remaining
@@ -54,13 +26,15 @@ XTreeSplit<TreeType>::XTreeSplit(const TreeType &other) :
  * new nodes into the tree, spliting the parent if necessary.
  */
 template<typename TreeType>
-void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
-                                         std::vector<bool>& relevels)
+void XTreeSplit::SplitLeafNode(TreeType *tree,std::vector<bool>& relevels)
 {
   // Convenience typedef.
   typedef typename TreeType::ElemType ElemType;
   typedef bound::HRectBound<metric::EuclideanDistance, ElemType> BoundType;
 
+  if (tree->Count() <= tree->MaxLeafSize())
+    return;
+
   // If we are splitting the root node, we need will do things differently so
   // that the constructor and other methods don't confuse the end user by giving
   // an address of another node.
@@ -72,9 +46,9 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     tree->Count() = 0;
     tree->NullifyData();
     // Because this was a leaf node, numChildren must be 0.
-    tree->Children()[(tree->NumChildren())++] = copy;
+    tree->children[(tree->NumChildren())++] = copy;
     assert(tree->NumChildren() == 1);
-    copy->Split().SplitLeafNode(copy, relevels);
+    XTreeSplit::SplitLeafNode(copy,relevels);
     return;
   }
 
@@ -92,7 +66,7 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     size_t p = tree->MaxLeafSize() * 0.3;
     if (p == 0)
     {
-      tree->Split().SplitLeafNode(tree, relevels);
+      XTreeSplit::SplitLeafNode(tree,relevels);
       return;
     }
 
@@ -102,8 +76,8 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     for (size_t i = 0; i < sorted.size(); i++)
     {
       sorted[i].first = tree->Metric().Evaluate(center,
-          tree->LocalDataset().col(i));
-      sorted[i].second = i;
+          tree->Dataset().col(tree->Point(i)));
+       sorted[i].second = i;
     }
 
     std::sort(sorted.begin(), sorted.end(), PairComp<ElemType>);
@@ -112,8 +86,9 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     for (size_t i = 0; i < p; i++)
     {
       // We start from the end of sorted.
-      pointIndices[i] = tree->Points()[sorted[sorted.size() - 1 - i].second];
-      root->DeletePoint(tree->Points()[sorted[sorted.size() - 1 - i].second],
+      pointIndices[i] = tree->Point(sorted[sorted.size() - 1 - i].second);
+
+      root->DeletePoint(tree->Point(sorted[sorted.size() - 1 - i].second),
           relevels);
     }
 
@@ -124,7 +99,7 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     }
 
 //    // If we went below min fill, delete this node and reinsert all points.
-//    if(tree->Count() < tree->MinLeafSize()) {
+//    if (tree->Count() < tree->MinLeafSize()) {
 //      std::vector<int> pointIndices(tree->Count());
 //      for(size_t i = 0; i < tree->Count(); i++) {
 //        pointIndices[i] = tree->Points()[i];
@@ -148,8 +123,9 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     ElemType axisScore = 0.0;
     // Since we only have points in the leaf nodes, we only need to sort once.
     std::vector<std::pair<ElemType, size_t>> sorted(tree->Count());
-    for (size_t i = 0; i < sorted.size(); i++) {
-      sorted[i].first = tree->LocalDataset().col(i)[j];
+    for (size_t i = 0; i < sorted.size(); i++)
+    {
+      sorted[i].first = tree->Dataset().col(tree->Point(i))[j];
       sorted[i].second = i;
     }
 
@@ -180,10 +156,10 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
       BoundType bound2(tree->Bound().Dim());
 
       for (size_t l = 0; l < cutOff; l++)
-        bound1 |= tree->LocalDataset().col(sorted[l].second);
+        bound1 |= tree->Dataset().col(tree->Point(sorted[l].second));
 
       for (size_t l = cutOff; l < tree->Count(); l++)
-        bound2 |= tree->LocalDataset().col(sorted[l].second);
+        bound2 |= tree->Dataset().col(tree->Point(sorted[l].second));
 
       ElemType area1 = bound1.Volume();
       ElemType area2 = bound2.Volume();
@@ -225,14 +201,16 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
   std::vector<std::pair<ElemType, size_t>> sorted(tree->Count());
   for (size_t i = 0; i < sorted.size(); i++)
   {
-    sorted[i].first = tree->LocalDataset().col(i)[bestAxis];
+    sorted[i].first = tree->Dataset().col(tree->Point(i))[bestAxis];
     sorted[i].second = i;
   }
 
   std::sort(sorted.begin(), sorted.end(), PairComp<ElemType>);
 
-  TreeType* treeOne = new TreeType(tree->Parent(), NormalNodeMaxNumChildren());
-  TreeType* treeTwo = new TreeType(tree->Parent(), NormalNodeMaxNumChildren());
+  TreeType* treeOne = new TreeType(tree->Parent(),
+                            tree->AuxiliaryInfo().NormalNodeMaxNumChildren());
+  TreeType* treeTwo = new TreeType(tree->Parent(),
+                            tree->AuxiliaryInfo().NormalNodeMaxNumChildren());
 
   // The leaf nodes should never have any overlap introduced by the above method
   // since a split axis is chosen and then points are assigned based on their
@@ -242,9 +220,9 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     for (size_t i = 0; i < tree->Count(); i++)
     {
       if (i < bestAreaIndexOnBestAxis + tree->MinLeafSize())
-        treeOne->InsertPoint(tree->Points()[sorted[i].second]);
+        treeOne->InsertPoint(tree->Point(sorted[i].second));
       else
-        treeTwo->InsertPoint(tree->Points()[sorted[i].second]);
+        treeTwo->InsertPoint(tree->Point(sorted[i].second));
     }
   }
   else
@@ -252,9 +230,9 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
     for (size_t i = 0; i < tree->Count(); i++)
     {
       if (i < bestOverlapIndexOnBestAxis + tree->MinLeafSize())
-        treeOne->InsertPoint(tree->Points()[sorted[i].second]);
+        treeOne->InsertPoint(tree->Point(sorted[i].second));
       else
-        treeTwo->InsertPoint(tree->Points()[sorted[i].second]);
+        treeTwo->InsertPoint(tree->Point(sorted[i].second));
     }
   }
 
@@ -263,27 +241,27 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
   size_t index = par->NumChildren();
   for (size_t i = 0; i < par->NumChildren(); i++)
   {
-    if (par->Children()[i] == tree)
+    if (par->children[i] == tree)
     {
       index = i;
       break;
     }
   }
   assert(index != par->NumChildren());
-  par->Children()[index] = treeOne;
-  par->Children()[par->NumChildren()++] = treeTwo;
+  par->children[index] = treeOne;
+  par->children[par->NumChildren()++] = treeTwo;
 
   // We now update the split history of each new node.
-  treeOne->Split().SplitHistory().history[bestAxis] = true;
-  treeOne->Split().SplitHistory().lastDimension = bestAxis;
-  treeTwo->Split().SplitHistory().history[bestAxis] = true;
-  treeTwo->Split().SplitHistory().lastDimension = bestAxis;
+  treeOne->AuxiliaryInfo().SplitHistory().history[bestAxis] = true;
+  treeOne->AuxiliaryInfo().SplitHistory().lastDimension = bestAxis;
+  treeTwo->AuxiliaryInfo().SplitHistory().history[bestAxis] = true;
+  treeTwo->AuxiliaryInfo().SplitHistory().lastDimension = bestAxis;
 
   // We only add one at a time, so we should only need to test for equality just
   // in case, we use an assert.
   assert(par->NumChildren() <= par->MaxNumChildren() + 1);
   if (par->NumChildren() == par->MaxNumChildren() + 1)
-    par->Split().SplitNonLeafNode(par, relevels);
+    XTreeSplit::SplitNonLeafNode(par,relevels);
 
   assert(treeOne->Parent()->NumChildren() <=
       treeOne->Parent()->MaxNumChildren());
@@ -305,8 +283,7 @@ void XTreeSplit<TreeType>::SplitLeafNode(TreeType* tree,
  * higher up the tree because they were already updated if necessary.
  */
 template<typename TreeType>
-bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
-                                            std::vector<bool>& relevels)
+bool XTreeSplit::SplitNonLeafNode(TreeType *tree,std::vector<bool>& relevels)
 {
   // Convenience typedef.
   typedef typename TreeType::ElemType ElemType;
@@ -323,8 +300,8 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
     copy->Parent() = tree;
     tree->NumChildren() = 0;
     tree->NullifyData();
-    tree->Children()[(tree->NumChildren())++] = copy;
-    copy->Split().SplitNonLeafNode(copy, relevels);
+    tree->children[(tree->NumChildren())++] = copy;
+    XTreeSplit::SplitNonLeafNode(copy,relevels);
     return true;
   }
 
@@ -339,7 +316,8 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
   std::vector<bool> axes(tree->Bound().Dim());
   std::vector<int> dimensionsLastUsed(tree->NumChildren());
   for (size_t i = 0; i < tree->NumChildren(); i++)
-    dimensionsLastUsed[i] = tree->Child(i).Split().SplitHistory().lastDimension;
+    dimensionsLastUsed[i] = 
+                    tree->Child(i).AuxiliaryInfo().SplitHistory().lastDimension;
   std::sort(dimensionsLastUsed.begin(), dimensionsLastUsed.end());
 
   size_t lastDim = dimensionsLastUsed[dimensionsLastUsed.size()/2];
@@ -350,7 +328,8 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
   {
     axes[i] = true;
     for (size_t j = 0; j < tree->NumChildren(); j++)
-      axes[i] = axes[i] & tree->Child(j).Split().SplitHistory().history[i];
+      axes[i] = axes[i] & 
+                tree->Child(j).AuxiliaryInfo().SplitHistory().history[i];
     if (axes[i] == true)
     {
       minOverlapSplitDimension = i;
@@ -363,7 +342,8 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
     {
       axes[i] = true;
       for (size_t j = 0; j < tree->NumChildren(); j++)
-        axes[i] = axes[i] & tree->Child(j).Split().SplitHistory().history[i];
+        axes[i] = axes[i] & 
+                  tree->Child(j).AuxiliaryInfo().SplitHistory().history[i];
       if (axes[i] == true)
       {
         minOverlapSplitDimension = i;
@@ -396,7 +376,7 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
     std::vector<std::pair<ElemType, size_t>> sorted(tree->NumChildren());
     for (size_t i = 0; i < sorted.size(); i++)
     {
-      sorted[i].first = tree->Children()[i]->Bound()[j].Lo();
+      sorted[i].first = tree->Child(i).Bound()[j].Lo();
       sorted[i].second = i;
     }
 
@@ -501,7 +481,7 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
     std::vector<std::pair<ElemType, size_t>> sorted(tree->NumChildren());
     for (size_t i = 0; i < sorted.size(); i++)
     {
-      sorted[i].first = tree->Children()[i]->Bound()[j].Hi();
+      sorted[i].first = tree->Child(i).Bound()[j].Hi();
       sorted[i].second = i;
     }
 
@@ -606,7 +586,7 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
   {
     for (size_t i = 0; i < sorted.size(); i++)
     {
-      sorted[i].first = tree->Children()[i]->Bound()[bestAxis].Lo();
+      sorted[i].first = tree->Child(i).Bound()[bestAxis].Lo();
       sorted[i].second = i;
     }
   }
@@ -614,7 +594,7 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
   {
     for (size_t i = 0; i < sorted.size(); i++)
     {
-      sorted[i].first = tree->Children()[i]->Bound()[bestAxis].Hi();
+      sorted[i].first = tree->Child(i).Bound()[bestAxis].Hi();
       sorted[i].second = i;
     }
   }
@@ -633,9 +613,9 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
       for (size_t i = 0; i < tree->NumChildren(); i++)
       {
         if (i < bestAreaIndexOnBestAxis + tree->MinNumChildren())
-          InsertNodeIntoTree(treeOne, tree->Children()[sorted[i].second]);
+          InsertNodeIntoTree(treeOne, tree->children[sorted[i].second]);
         else
-          InsertNodeIntoTree(treeTwo, tree->Children()[sorted[i].second]);
+          InsertNodeIntoTree(treeTwo, tree->children[sorted[i].second]);
       }
     }
     else
@@ -648,9 +628,9 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
       for (size_t i = 0; i < tree->NumChildren(); i++)
       {
         if (i < bestOverlapIndexOnBestAxis + tree->MinNumChildren())
-          InsertNodeIntoTree(treeOne, tree->Children()[sorted[i].second]);
+          InsertNodeIntoTree(treeOne, tree->children[sorted[i].second]);
         else
-          InsertNodeIntoTree(treeTwo, tree->Children()[sorted[i].second]);
+          InsertNodeIntoTree(treeTwo, tree->children[sorted[i].second]);
       }
     }
     else
@@ -671,7 +651,7 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
       {
         for (size_t i = 0; i < sorted2.size(); i++)
         {
-          sorted2[i].first = tree->Children()[i]->Bound()[bestAxis].Hi();
+          sorted2[i].first = tree->Child(i).Bound()[bestAxis].Hi();
           sorted2[i].second = i;
         }
       }
@@ -679,7 +659,7 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
       {
         for (size_t i = 0; i < sorted2.size(); i++)
         {
-          sorted2[i].first = tree->Children()[i]->Bound()[bestAxis].Lo();
+          sorted2[i].first = tree->Child(i).Bound()[bestAxis].Lo();
           sorted2[i].second = i;
         }
       }
@@ -688,9 +668,9 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
       for (size_t i = 0; i < tree->NumChildren(); i++)
       {
         if (i < bestIndexMinOverlapSplit + tree->MinNumChildren())
-          InsertNodeIntoTree(treeOne, tree->Children()[sorted[i].second]);
+          InsertNodeIntoTree(treeOne, tree->children[sorted2[i].second]);
         else
-          InsertNodeIntoTree(treeTwo, tree->Children()[sorted[i].second]);
+          InsertNodeIntoTree(treeTwo, tree->children[sorted2[i].second]);
       }
     }
     else
@@ -708,13 +688,13 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
           (tree->Parent()->NumChildren() == 1))
       {
         // We make the root a supernode instead.
-        tree->Parent()->MaxNumChildren() = tree->MaxNumChildren() +
-            NormalNodeMaxNumChildren();
-        tree->Parent()->Children().resize(tree->Parent()->MaxNumChildren() + 1);
+        tree->Parent()->MaxNumChildren() = tree->MaxNumChildren() + 
+                              tree->AuxiliaryInfo().NormalNodeMaxNumChildren();
+        tree->Parent()->children.resize(tree->Parent()->MaxNumChildren() + 1);
         tree->Parent()->NumChildren() = tree->NumChildren();
         for (size_t i = 0; i < tree->NumChildren(); i++)
         {
-          tree->Parent()->Children()[i] = tree->Children()[i];
+          tree->Parent()->children[i] = tree->children[i];
           tree->Child(i).Parent() = tree->Parent();
         }
 
@@ -726,8 +706,9 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
       }
 
       // If we don't have to worry about the root, we just enlarge this node.
-      tree->MaxNumChildren() += NormalNodeMaxNumChildren();
-      tree->Children().resize(tree->MaxNumChildren() + 1);
+      tree->MaxNumChildren() += 
+                              tree->AuxiliaryInfo().NormalNodeMaxNumChildren();
+      tree->children.resize(tree->MaxNumChildren() + 1);
       for (size_t i = 0; i < tree->NumChildren(); i++)
         tree->Child(i).Parent() = tree;
 
@@ -739,25 +720,25 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
   }
 
   // Update the split history of each child.
-  treeOne->Split().SplitHistory().history[bestAxis] = true;
-  treeOne->Split().SplitHistory().lastDimension = bestAxis;
-  treeTwo->Split().SplitHistory().history[bestAxis] = true;
-  treeTwo->Split().SplitHistory().lastDimension = bestAxis;
+  treeOne->AuxiliaryInfo().SplitHistory().history[bestAxis] = true;
+  treeOne->AuxiliaryInfo().SplitHistory().lastDimension = bestAxis;
+  treeTwo->AuxiliaryInfo().SplitHistory().history[bestAxis] = true;
+  treeTwo->AuxiliaryInfo().SplitHistory().lastDimension = bestAxis;
 
   // Remove this node and insert treeOne and treeTwo
   TreeType* par = tree->Parent();
   size_t index = 0;
   for (size_t i = 0; i < par->NumChildren(); i++)
   {
-    if (par->Children()[i] == tree)
+    if (par->children[i] == tree)
     {
       index = i;
       break;
     }
   }
 
-  par->Children()[index] = treeOne;
-  par->Children()[par->NumChildren()++] = treeTwo;
+  par->children[index] = treeOne;
+  par->children[par->NumChildren()++] = treeTwo;
 
   // we only add one at a time, so we should only need to test for equality
   // just in case, we use an assert.
@@ -768,16 +749,14 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
   assert(par->NumChildren() <= par->MaxNumChildren() + 1);
 
   if (par->NumChildren() == par->MaxNumChildren() + 1)
-  {
-    par->Split().SplitNonLeafNode(par, relevels);
-  }
+    XTreeSplit::SplitNonLeafNode(par,relevels);
 
   // We have to update the children of each of these new nodes so that they
   // record the correct parent.
   for (size_t i = 0; i < treeOne->NumChildren(); i++)
-    treeOne->Children()[i]->Parent() = treeOne;
+    treeOne->Child(i).Parent() = treeOne;
   for (size_t i = 0; i < treeTwo->NumChildren(); i++)
-    treeTwo->Children()[i]->Parent() = treeTwo;
+    treeTwo->Child(i).Parent() = treeTwo;
 
   assert(treeOne->Parent()->NumChildren() <=
       treeOne->Parent()->MaxNumChildren());
@@ -798,27 +777,14 @@ bool XTreeSplit<TreeType>::SplitNonLeafNode(TreeType* tree,
  * numberOfChildren.
  */
 template<typename TreeType>
-void XTreeSplit<TreeType>::InsertNodeIntoTree(TreeType* destTree, TreeType* srcNode)
+void XTreeSplit::InsertNodeIntoTree(TreeType* destTree, TreeType* srcNode)
 {
   destTree->Bound() |= srcNode->Bound();
-  destTree->Children()[destTree->NumChildren()] = srcNode;
+  destTree->numDescendants += srcNode->numDescendants;
+  destTree->children[destTree->NumChildren()] = srcNode;
   destTree->NumChildren()++;
 }
 
-/**
- * Serialize the split.
- */
-template<typename TreeType>
-template<typename Archive>
-void XTreeSplit<TreeType>::Serialize(Archive& ar,const unsigned int /* version */)
-{
-  using data::CreateNVP;
-
-  ar & CreateNVP(normalNodeMaxNumChildren, "normalNodeMaxNumChildren");
-  ar & CreateNVP(splitHistory, "splitHistory");
-
-}
-
 } // namespace tree
 } // namespace mlpack
 
