@@ -26,8 +26,6 @@
  *   output coding}
  * @endcode
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -71,27 +69,28 @@ PROGRAM_INFO("AdaBoost", "This program implements the AdaBoost (or Adaptive "
     "a file specified by the --output_model_file (-M) parameter.");
 
 // Input for training.
-PARAM_STRING("training_file", "A file containing the training set.", "t", "");
-PARAM_STRING("labels_file", "A file containing labels for the training set.",
-  "l", "");
+PARAM_STRING_IN("training_file", "A file containing the training set.", "t",
+    "");
+PARAM_STRING_IN("labels_file", "A file containing labels for the training set.",
+    "l", "");
 
 // Loading/saving of a model.
-PARAM_STRING("input_model_file", "File containing input AdaBoost model.", "m",
-    "");
-PARAM_STRING("output_model_file", "File to save trained AdaBoost model to.",
-    "M", "");
+PARAM_STRING_IN("input_model_file", "File containing input AdaBoost model.",
+    "m", "");
+PARAM_STRING_OUT("output_model_file", "File to save trained AdaBoost model to.",
+    "M");
 
 // Classification options.
-PARAM_STRING("test_file", "A file containing the test set.", "T", "");
-PARAM_STRING("output_file", "The file in which the predicted labels for the "
-    "test set will be written.", "o", "");
+PARAM_STRING_IN("test_file", "A file containing the test set.", "T", "");
+PARAM_STRING_OUT("output_file", "The file in which the predicted labels for the"
+    " test set will be written.", "o");
 
 // Training options.
-PARAM_INT("iterations", "The maximum number of boosting iterations to be run. "
-    "(0 will run until convergence.)", "i", 1000);
-PARAM_DOUBLE("tolerance", "The tolerance for change in values of the weighted "
-    "error during training.", "e", 1e-10);
-PARAM_STRING("weak_learner", "The type of weak learner to use: "
+PARAM_INT_IN("iterations", "The maximum number of boosting iterations to be run"
+    " (0 will run until convergence.)", "i", 1000);
+PARAM_DOUBLE_IN("tolerance", "The tolerance for change in values of the "
+    "weighted error during training.", "e", 1e-10);
+PARAM_STRING_IN("weak_learner", "The type of weak learner to use: "
     "'decision_stump', or 'perceptron'.", "w", "decision_stump");
 
 /**
