@@ -4,8 +4,6 @@
  *
  * Executable for Kernel PCA.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -36,11 +34,9 @@ PROGRAM_INFO("Kernel Principal Components Analysis",
     "For the case where a linear kernel is used, this reduces to regular "
     "PCA."
     "\n\n"
-
     "For example, the following will perform KPCA on the 'input.csv' file using"
     " the gaussian kernel and store the transformed date in the "
     "'transformed.csv' file."
-
     "\n\n"
     "$ kernel_pca -i input.csv -k gaussian -o transformed.csv"
     "\n\n"
@@ -79,12 +75,12 @@ PROGRAM_INFO("Kernel Principal Components Analysis",
     " for the nystr\u00F6m method can be chosen from the following list: kmeans,"
     " random, ordered.");
 
-PARAM_STRING_REQ("input_file", "Input dataset to perform KPCA on.", "i");
-PARAM_STRING_REQ("output_file", "File to save modified dataset to.", "o");
-PARAM_STRING_REQ("kernel", "The kernel to use; see the above documentation for "
-    "the list of usable kernels.", "k");
+PARAM_STRING_IN_REQ("input_file", "Input dataset to perform KPCA on.", "i");
+PARAM_STRING_OUT("output_file", "File to save modified dataset to.", "o");
+PARAM_STRING_IN_REQ("kernel", "The kernel to use; see the above documentation "
+    "for the list of usable kernels.", "k");
 
-PARAM_INT("new_dimensionality", "If not 0, reduce the dimensionality of "
+PARAM_INT_IN("new_dimensionality", "If not 0, reduce the dimensionality of "
     "the output dataset by ignoring the dimensions with the smallest "
     "eigenvalues.", "d", 0);
 
@@ -93,15 +89,15 @@ PARAM_FLAG("center", "If set, the transformed data will be centered about the "
 
 PARAM_FLAG("nystroem_method", "If set, the nystroem method will be used.", "n");
 
-PARAM_STRING("sampling", "Sampling scheme to use for the nystroem method: "
+PARAM_STRING_IN("sampling", "Sampling scheme to use for the nystroem method: "
     "'kmeans', 'random', 'ordered'", "s", "kmeans");
 
-PARAM_DOUBLE("kernel_scale", "Scale, for 'hyptan' kernel.", "S", 1.0);
-PARAM_DOUBLE("offset", "Offset, for 'hyptan' and 'polynomial' kernels.", "O",
+PARAM_DOUBLE_IN("kernel_scale", "Scale, for 'hyptan' kernel.", "S", 1.0);
+PARAM_DOUBLE_IN("offset", "Offset, for 'hyptan' and 'polynomial' kernels.", "O",
     0.0);
-PARAM_DOUBLE("bandwidth", "Bandwidth, for 'gaussian' and 'laplacian' kernels.",
-    "b", 1.0);
-PARAM_DOUBLE("degree", "Degree of polynomial, for 'polynomial' kernel.", "D",
+PARAM_DOUBLE_IN("bandwidth", "Bandwidth, for 'gaussian' and 'laplacian' "
+    "kernels.", "b", 1.0);
+PARAM_DOUBLE_IN("degree", "Degree of polynomial, for 'polynomial' kernel.", "D",
     1.0);
 
 //! Run RunKPCA on the specified dataset for the given kernel type.
