@@ -7,8 +7,6 @@
  * probably limited to the case where k is close to the number of points in the
  * dataset, and the number of iterations of the k-means algorithm will be few.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -108,17 +106,17 @@ double DualTreeKMeans<MetricType, MatType, TreeType>::Iterate(
   std::vector<size_t> oldFromNewCentroids;
   Tree* centroidTree = BuildTree<Tree>(centroids, oldFromNewCentroids);
 
+  // Find the nearest neighbors of each of the clusters.  We have to make our
+  // own TreeType, which is a little bit abuse, but we know for sure the
+  // TreeStatType we have will work.
+  neighbor::NeighborSearch<neighbor::NearestNeighborSort, MetricType, MatType,
+      NNSTreeType> nns(std::move(*centroidTree));
+
   // Reset information in the tree, if we need to.
   if (iteration > 0)
   {
     Timer::Start("knn");
 
-    // Find the nearest neighbors of each of the clusters.  We have to make our
-    // own TreeType, which is a little bit abuse, but we know for sure the
-    // TreeStatType we have will work.
-    neighbor::NeighborSearch<neighbor::NearestNeighborSort, MetricType, MatType,
-        NNSTreeType> nns(centroidTree);
-
     // If the tree maps points, we need an intermediate result matrix.
     arma::mat* interclusterDistancesTemp =
         (tree::TreeTraits<Tree>::RearrangesDataset) ?
@@ -155,8 +153,9 @@ double DualTreeKMeans<MetricType, MatType, TreeType>::Iterate(
   // We won't use the KNN class here because we have our own set of rules.
   lastIterationCentroids = centroids;
   typedef DualTreeKMeansRules<MetricType, Tree> RuleType;
-  RuleType rules(centroidTree->Dataset(), dataset, assignments, upperBounds,
-      lowerBounds, metric, prunedPoints, oldFromNewCentroids, visited);
+  RuleType rules(nns.ReferenceTree().Dataset(), dataset, assignments,
+      upperBounds, lowerBounds, metric, prunedPoints, oldFromNewCentroids,
+      visited);
 
   typename Tree::template BreadthFirstDualTreeTraverser<RuleType>
       traverser(rules);
@@ -167,7 +166,7 @@ double DualTreeKMeans<MetricType, MatType, TreeType>::Iterate(
 
   // Set the number of pruned centroids in the root to 0.
   tree->Stat().Pruned() = 0;
-  traverser.Traverse(*tree, *centroidTree);
+  traverser.Traverse(*tree, nns.ReferenceTree());
   distanceCalculations += rules.BaseCases() + rules.Scores();
 
   Timer::Start("tree_mod");
