@@ -5,8 +5,6 @@
  * Implementation of the SortPolicy class for NeighborSearch; in this case, the
  * furthest neighbors are those that are most important.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -29,24 +27,6 @@ namespace neighbor {
 class FurthestNeighborSort
 {
  public:
-  /**
-   * Return the index in the vector where the new distance should be inserted,
-   * or size_t() - 1 if it should not be inserted (i.e. if it is not any better
-   * than any of the existing points in the list).  The list should be sorted
-   * such that the best point is the first in the list.  The actual insertion is
-   * not performed.
-   *
-   * @param list Vector of existing distance points, sorted such that the best
-   *     point is the first in the list.
-   * @param new_distance Distance to try to insert.
-   *
-   * @return size_t containing the position to insert into, or (size_t() - 1)
-   *     if the new distance should not be inserted.
-   */
-  static size_t SortDistance(const arma::vec& list,
-                             const arma::Col<size_t>& indices,
-                             double newDistance);
-
   /**
    * Return whether or not value is "better" than ref.  In this case, that means
    * that the value is greater than or equal to the reference.
@@ -119,6 +99,26 @@ class FurthestNeighborSort
                                         const TreeType* referenceNode,
                                         const double pointToCenterDistance);
 
+  /**
+   * Return the best child according to this sort policy. In this case it will
+   * return the one with the maximum distance.
+   */
+  template<typename VecType, typename TreeType>
+  static size_t GetBestChild(const VecType& queryPoint, TreeType& referenceNode)
+  {
+    return referenceNode.GetFurthestChild(queryPoint);
+  };
+
+  /**
+   * Return the best child according to this sort policy. In this case it will
+   * return the one with the maximum distance.
+   */
+  template<typename TreeType>
+  static size_t GetBestChild(const TreeType& queryNode, TreeType& referenceNode)
+  {
+    return referenceNode.GetFurthestChild(queryNode);
+  };
+
   /**
    * Return what should represent the worst possible distance with this
    * particular sort policy.  In our case, this should be the minimum possible
@@ -152,6 +152,48 @@ class FurthestNeighborSort
    */
   static inline double CombineWorst(const double a, const double b)
   { return std::max(a - b, 0.0); }
+
+  /**
+   * Return the given value relaxed.
+   *
+   * @param value Value to relax.
+   * @param epsilon Relative error (non-negative).
+   *
+   * @return double Value relaxed.
+   */
+  static inline double Relax(const double value, const double epsilon)
+  {
+    if (value == 0)
+      return 0;
+    if (value == DBL_MAX || epsilon >= 1)
+      return DBL_MAX;
+    return (1 / (1 - epsilon)) * value;
+  }
+
+  /**
+   * Convert the given distance to a score.  Lower scores are better, but for
+   * furthest neighbor search, larger distances are better.  Therefore we must
+   * invert the given distance.
+   */
+  static inline double ConvertToScore(const double distance)
+  {
+    if (distance == DBL_MAX)
+      return 0.0;
+    else if (distance == 0.0)
+      return DBL_MAX;
+    else
+      return (1.0 / distance);
+  }
+
+  /**
+   * Convert the given score back to a distance.  This is the inverse of the
+   * operation of converting a distance to a score, and again, for furthest
+   * neighbor search, corresponds to inverting the value.
+   */
+  static inline double ConvertToDistance(const double score)
+  {
+    return ConvertToScore(score);
+  }
 };
 
 } // namespace neighbor
