@@ -5,8 +5,6 @@
  * Implementation of the SortPolicy class for NeighborSearch; in this case, the
  * nearest neighbors are those that are most important.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -33,24 +31,6 @@ namespace neighbor {
 class NearestNeighborSort
 {
  public:
-  /**
-   * Return the index in the vector where the new distance should be inserted,
-   * or (size_t() - 1) if it should not be inserted (i.e. if it is not any
-   * better than any of the existing points in the list).  The list should be
-   * sorted such that the best point is the first in the list.  The actual
-   * insertion is not performed.
-   *
-   * @param list Vector of existing distance points, sorted such that the best
-   *     point is first in the list.
-   * @param new_distance Distance to try to insert
-   *
-   * @return size_t containing the position to insert into, or (size_t() - 1)
-   *     if the new distance should not be inserted.
-   */
-  static size_t SortDistance(const arma::vec& list,
-                             const arma::Col<size_t>& indices,
-                             double newDistance);
-
   /**
    * Return whether or not value is "better" than ref.  In this case, that means
    * that the value is less than or equal to the reference.
@@ -102,6 +82,7 @@ class NearestNeighborSort
                                        const TreeType* referenceNode,
                                        const TreeType* referenceChildNode,
                                        const double centerToCenterDistance);
+
   /**
    * Return the best possible distance between a node and a point.  In our case,
    * this is the minimum distance between the tree node and the point using the
@@ -122,6 +103,26 @@ class NearestNeighborSort
                                         const TreeType* referenceNode,
                                         const double pointToCenterDistance);
 
+  /**
+   * Return the best child according to this sort policy. In this case it will
+   * return the one with the minimum distance.
+   */
+  template<typename VecType, typename TreeType>
+  static size_t GetBestChild(const VecType& queryPoint, TreeType& referenceNode)
+  {
+    return referenceNode.GetNearestChild(queryPoint);
+  };
+
+  /**
+   * Return the best child according to this sort policy. In this case it will
+   * return the one with the minimum distance.
+   */
+  template<typename TreeType>
+  static size_t GetBestChild(const TreeType& queryNode, TreeType& referenceNode)
+  {
+    return referenceNode.GetNearestChild(queryNode);
+  };
+
   /**
    * Return what should represent the worst possible distance with this
    * particular sort policy.  In our case, this should be the maximum possible
@@ -157,6 +158,41 @@ class NearestNeighborSort
       return DBL_MAX;
     return a + b;
   }
+
+  /**
+   * Return the given value relaxed.
+   *
+   * @param value Value to relax.
+   * @param epsilon Relative error (non-negative).
+   *
+   * @return double Value relaxed.
+   */
+  static inline double Relax(const double value, const double epsilon)
+  {
+    if (value == DBL_MAX)
+      return DBL_MAX;
+    return (1 / (1 + epsilon)) * value;
+  }
+
+  /**
+   * Convert the given distance into a score.  Lower scores are better, so in
+   * the case of nearest neighbor sort where lower distances are better, we just
+   * return the distance.
+   */
+  static inline double ConvertToScore(const double distance)
+  {
+    return distance;
+  }
+
+  /**
+   * Convert the given score to a distance.  This is the inverse of the
+   * operation provided by ConvertToScore().  For nearest neighbor search, there
+   * is no need for any change.
+   */
+  static inline double ConvertToDistance(const double score)
+  {
+    return score;
+  }
 };
 
 } // namespace neighbor
