@@ -4,8 +4,6 @@
  *
  * Implementation of the range search model class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -23,13 +21,22 @@ using namespace mlpack::range;
  */
 RSModel::RSModel(TreeTypes treeType, bool randomBasis) :
     treeType(treeType),
+    leafSize(0),
     randomBasis(randomBasis),
     kdTreeRS(NULL),
     coverTreeRS(NULL),
     rTreeRS(NULL),
     rStarTreeRS(NULL),
     ballTreeRS(NULL),
-    xTreeRS(NULL)
+    xTreeRS(NULL),
+    hilbertRTreeRS(NULL),
+    rPlusTreeRS(NULL),
+    rPlusPlusTreeRS(NULL),
+    vpTreeRS(NULL),
+    rpTreeRS(NULL),
+    maxRPTreeRS(NULL),
+    ubTreeRS(NULL),
+    octreeRS(NULL)
 {
   // Nothing to do.
 }
@@ -52,6 +59,8 @@ void RSModel::BuildModel(arma::mat&& referenceSet,
     math::RandomBasis(q, referenceSet.n_rows);
   }
 
+  this->leafSize = leafSize;
+
   // Clean memory, if necessary.
   CleanMemory();
 
@@ -129,6 +138,63 @@ void RSModel::BuildModel(arma::mat&& referenceSet,
       xTreeRS = new RSType<tree::XTree>(move(referenceSet), naive,
           singleMode);
       break;
+
+    case HILBERT_R_TREE:
+      hilbertRTreeRS = new RSType<tree::HilbertRTree>(move(referenceSet), naive,
+          singleMode);
+      break;
+
+    case R_PLUS_TREE:
+      rPlusTreeRS = new RSType<tree::RPlusTree>(move(referenceSet), naive,
+          singleMode);
+      break;
+
+    case R_PLUS_PLUS_TREE:
+      rPlusPlusTreeRS = new RSType<tree::RPlusPlusTree>(move(referenceSet),
+          naive, singleMode);
+      break;
+
+    case VP_TREE:
+      vpTreeRS = new RSType<tree::VPTree>(move(referenceSet), naive,
+          singleMode);
+      break;
+
+    case RP_TREE:
+      rpTreeRS = new RSType<tree::RPTree>(move(referenceSet), naive,
+          singleMode);
+      break;
+
+    case MAX_RP_TREE:
+      maxRPTreeRS = new RSType<tree::MaxRPTree>(move(referenceSet),
+          naive, singleMode);
+      break;
+
+    case UB_TREE:
+      ubTreeRS = new RSType<tree::UBTree>(move(referenceSet),
+          naive, singleMode);
+      break;
+
+    case OCTREE:
+      // If necessary, build the octree.
+      if (naive)
+      {
+        octreeRS = new RSType<tree::Octree>(move(referenceSet), naive,
+            singleMode);
+      }
+      else
+      {
+        vector<size_t> oldFromNewReferences;
+        RSType<tree::Octree>::Tree* octree =
+            new RSType<tree::Octree>::Tree(move(referenceSet),
+            oldFromNewReferences, leafSize);
+        octreeRS = new RSType<tree::Octree>(octree, singleMode);
+
+        // Give the model ownership of the tree and the mappings.
+        octreeRS->treeOwner = true;
+        octreeRS->oldFromNewReferences = move(oldFromNewReferences);
+      }
+
+      break;
   }
 
   if (!naive)
@@ -238,6 +304,66 @@ void RSModel::Search(arma::mat&& querySet,
     case X_TREE:
       xTreeRS->Search(querySet, range, neighbors, distances);
       break;
+
+    case HILBERT_R_TREE:
+      hilbertRTreeRS->Search(querySet, range, neighbors, distances);
+      break;
+
+    case R_PLUS_TREE:
+      rPlusTreeRS->Search(querySet, range, neighbors, distances);
+      break;
+
+    case R_PLUS_PLUS_TREE:
+      rPlusPlusTreeRS->Search(querySet, range, neighbors, distances);
+      break;
+
+    case VP_TREE:
+      vpTreeRS->Search(querySet, range, neighbors, distances);
+      break;
+
+    case RP_TREE:
+      rpTreeRS->Search(querySet, range, neighbors, distances);
+      break;
+
+    case MAX_RP_TREE:
+      maxRPTreeRS->Search(querySet, range, neighbors, distances);
+      break;
+
+    case UB_TREE:
+      ubTreeRS->Search(querySet, range, neighbors, distances);
+      break;
+
+    case OCTREE:
+      if (!octreeRS->Naive() && !octreeRS->SingleMode())
+      {
+        // Build a query tree and search.
+        Timer::Start("tree_building");
+        Log::Info << "Building query tree..." << endl;
+        vector<size_t> oldFromNewQueries;
+        RSType<tree::Octree>::Tree queryTree(move(querySet), oldFromNewQueries,
+            leafSize);
+        Log::Info << "Tree built." << endl;
+        Timer::Stop("tree_building");
+
+        vector<vector<size_t>> neighborsOut;
+        vector<vector<double>> distancesOut;
+        octreeRS->Search(&queryTree, range, neighborsOut, distancesOut);
+
+        // Remap the query points.
+        neighbors.resize(queryTree.Dataset().n_cols);
+        distances.resize(queryTree.Dataset().n_cols);
+        for (size_t i = 0; i < queryTree.Dataset().n_cols; ++i)
+        {
+          neighbors[oldFromNewQueries[i]] = neighborsOut[i];
+          distances[oldFromNewQueries[i]] = distancesOut[i];
+        }
+      }
+      else
+      {
+        // Search without building a second tree.
+        octreeRS->Search(querySet, range, neighbors, distances);
+      }
+      break;
   }
 }
 
@@ -280,6 +406,38 @@ void RSModel::Search(const math::Range& range,
     case X_TREE:
       xTreeRS->Search(range, neighbors, distances);
       break;
+
+    case HILBERT_R_TREE:
+      hilbertRTreeRS->Search(range, neighbors, distances);
+      break;
+
+    case R_PLUS_TREE:
+      rPlusTreeRS->Search(range, neighbors, distances);
+      break;
+
+    case R_PLUS_PLUS_TREE:
+      rPlusPlusTreeRS->Search(range, neighbors, distances);
+      break;
+
+    case VP_TREE:
+      vpTreeRS->Search(range, neighbors, distances);
+      break;
+
+    case RP_TREE:
+      rpTreeRS->Search(range, neighbors, distances);
+      break;
+
+    case MAX_RP_TREE:
+      maxRPTreeRS->Search(range, neighbors, distances);
+      break;
+
+    case UB_TREE:
+      ubTreeRS->Search(range, neighbors, distances);
+      break;
+
+    case OCTREE:
+      octreeRS->Search(range, neighbors, distances);
+      break;
   }
 }
 
@@ -300,6 +458,22 @@ std::string RSModel::TreeName() const
       return "ball tree";
     case X_TREE:
       return "X tree";
+    case HILBERT_R_TREE:
+      return "Hilbert R tree";
+    case R_PLUS_TREE:
+      return "R+ tree";
+    case R_PLUS_PLUS_TREE:
+      return "R++ tree";
+    case VP_TREE:
+      return "vantage point tree";
+    case RP_TREE:
+      return "random projection tree (mean split)";
+    case MAX_RP_TREE:
+      return "random projection tree (max split)";
+    case UB_TREE:
+      return "UB tree";
+    case OCTREE:
+      return "octree";
     default:
       return "unknown tree";
   }
@@ -308,18 +482,20 @@ std::string RSModel::TreeName() const
 // Clean memory.
 void RSModel::CleanMemory()
 {
-  if (kdTreeRS)
-    delete kdTreeRS;
-  if (coverTreeRS)
-    delete coverTreeRS;
-  if (rTreeRS)
-    delete rTreeRS;
-  if (rStarTreeRS)
-    delete rStarTreeRS;
-  if (ballTreeRS)
-    delete ballTreeRS;
-  if (xTreeRS)
-    delete xTreeRS;
+  delete kdTreeRS;
+  delete coverTreeRS;
+  delete rTreeRS;
+  delete rStarTreeRS;
+  delete ballTreeRS;
+  delete xTreeRS;
+  delete hilbertRTreeRS;
+  delete rPlusTreeRS;
+  delete rPlusPlusTreeRS;
+  delete vpTreeRS;
+  delete rpTreeRS;
+  delete maxRPTreeRS;
+  delete ubTreeRS;
+  delete octreeRS;
 
   kdTreeRS = NULL;
   coverTreeRS = NULL;
@@ -327,4 +503,12 @@ void RSModel::CleanMemory()
   rStarTreeRS = NULL;
   ballTreeRS = NULL;
   xTreeRS = NULL;
+  hilbertRTreeRS = NULL;
+  rPlusTreeRS = NULL;
+  rPlusPlusTreeRS = NULL;
+  vpTreeRS = NULL;
+  rpTreeRS = NULL;
+  maxRPTreeRS = NULL;
+  ubTreeRS = NULL;
+  octreeRS = NULL;
 }
