@@ -4,8 +4,6 @@
  *
  * Implementation of the RAModel class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -29,7 +27,12 @@ RAModel<SortPolicy>::RAModel(const TreeTypes treeType, const bool randomBasis) :
     coverTreeRA(NULL),
     rTreeRA(NULL),
     rStarTreeRA(NULL),
-    xTreeRA(NULL)
+    xTreeRA(NULL),
+    hilbertRTreeRA(NULL),
+    rPlusTreeRA(NULL),
+    rPlusPlusTreeRA(NULL),
+    ubTreeRA(NULL),
+    octreeRA(NULL)
 {
   // Nothing to do.
 }
@@ -37,16 +40,16 @@ RAModel<SortPolicy>::RAModel(const TreeTypes treeType, const bool randomBasis) :
 template<typename SortPolicy>
 RAModel<SortPolicy>::~RAModel()
 {
-  if (kdTreeRA)
-    delete kdTreeRA;
-  if (coverTreeRA)
-    delete coverTreeRA;
-  if (rTreeRA)
-    delete rTreeRA;
-  if (rStarTreeRA)
-    delete rStarTreeRA;
-  if (xTreeRA)
-    delete xTreeRA;
+  delete kdTreeRA;
+  delete coverTreeRA;
+  delete rTreeRA;
+  delete rStarTreeRA;
+  delete xTreeRA;
+  delete hilbertRTreeRA;
+  delete rPlusTreeRA;
+  delete rPlusPlusTreeRA;
+  delete ubTreeRA;
+  delete octreeRA;
 }
 
 template<typename SortPolicy>
@@ -61,16 +64,16 @@ void RAModel<SortPolicy>::Serialize(Archive& ar,
   // This should never happen, but just in case, be clean with memory.
   if (Archive::is_loading::value)
   {
-    if (kdTreeRA)
-      delete kdTreeRA;
-    if (coverTreeRA)
-      delete coverTreeRA;
-    if (rTreeRA)
-      delete rTreeRA;
-    if (rStarTreeRA)
-      delete rStarTreeRA;
-    if (xTreeRA)
-      delete xTreeRA;
+    delete kdTreeRA;
+    delete coverTreeRA;
+    delete rTreeRA;
+    delete rStarTreeRA;
+    delete xTreeRA;
+    delete hilbertRTreeRA;
+    delete rPlusTreeRA;
+    delete rPlusPlusTreeRA;
+    delete ubTreeRA;
+    delete octreeRA;
 
     // Set all the pointers to NULL.
     kdTreeRA = NULL;
@@ -78,6 +81,10 @@ void RAModel<SortPolicy>::Serialize(Archive& ar,
     rTreeRA = NULL;
     rStarTreeRA = NULL;
     xTreeRA = NULL;
+    hilbertRTreeRA = NULL;
+    rPlusPlusTreeRA = NULL;
+    rPlusTreeRA = NULL;
+    ubTreeRA = NULL;
   }
 
   // We only need to serialize one of the kRANN objects.
@@ -98,6 +105,21 @@ void RAModel<SortPolicy>::Serialize(Archive& ar,
     case X_TREE:
       ar & data::CreateNVP(xTreeRA, "ra_model");
       break;
+    case HILBERT_R_TREE:
+      ar & data::CreateNVP(hilbertRTreeRA, "ra_model");
+      break;
+    case R_PLUS_TREE:
+      ar & data::CreateNVP(rPlusTreeRA, "ra_model");
+      break;
+    case R_PLUS_PLUS_TREE:
+      ar & data::CreateNVP(rPlusPlusTreeRA, "ra_model");
+      break;
+    case UB_TREE:
+      ar & data::CreateNVP(ubTreeRA, "ra_model");
+      break;
+    case OCTREE:
+      ar & data::CreateNVP(octreeRA, "ra_model");
+      break;
   }
 }
 
@@ -114,6 +136,16 @@ const arma::mat& RAModel<SortPolicy>::Dataset() const
     return rStarTreeRA->ReferenceSet();
   else if (xTreeRA)
     return xTreeRA->ReferenceSet();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->ReferenceSet();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->ReferenceSet();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->ReferenceSet();
+  else if (ubTreeRA)
+    return ubTreeRA->ReferenceSet();
+  else if (octreeRA)
+    return octreeRA->ReferenceSet();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -132,6 +164,16 @@ bool RAModel<SortPolicy>::Naive() const
     return rStarTreeRA->Naive();
   else if (xTreeRA)
     return xTreeRA->Naive();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->Naive();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->Naive();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->Naive();
+  else if (ubTreeRA)
+    return ubTreeRA->Naive();
+  else if (octreeRA)
+    return octreeRA->Naive();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -150,6 +192,16 @@ bool& RAModel<SortPolicy>::Naive()
     return rStarTreeRA->Naive();
   else if (xTreeRA)
     return xTreeRA->Naive();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->Naive();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->Naive();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->Naive();
+  else if (ubTreeRA)
+    return ubTreeRA->Naive();
+  else if (octreeRA)
+    return octreeRA->Naive();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -168,6 +220,16 @@ bool RAModel<SortPolicy>::SingleMode() const
     return rStarTreeRA->SingleMode();
   else if (xTreeRA)
     return xTreeRA->SingleMode();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->SingleMode();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->SingleMode();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->SingleMode();
+  else if (ubTreeRA)
+    return ubTreeRA->SingleMode();
+  else if (octreeRA)
+    return octreeRA->SingleMode();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -186,6 +248,16 @@ bool& RAModel<SortPolicy>::SingleMode()
     return rStarTreeRA->SingleMode();
   else if (xTreeRA)
     return xTreeRA->SingleMode();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->SingleMode();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->SingleMode();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->SingleMode();
+  else if (ubTreeRA)
+    return ubTreeRA->SingleMode();
+  else if (octreeRA)
+    return octreeRA->SingleMode();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -204,6 +276,16 @@ double RAModel<SortPolicy>::Tau() const
     return rStarTreeRA->Tau();
   else if (xTreeRA)
     return xTreeRA->Tau();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->Tau();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->Tau();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->Tau();
+  else if (ubTreeRA)
+    return ubTreeRA->Tau();
+  else if (octreeRA)
+    return octreeRA->Tau();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -222,6 +304,16 @@ double& RAModel<SortPolicy>::Tau()
     return rStarTreeRA->Tau();
   else if (xTreeRA)
     return xTreeRA->Tau();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->Tau();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->Tau();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->Tau();
+  else if (ubTreeRA)
+    return ubTreeRA->Tau();
+  else if (octreeRA)
+    return octreeRA->Tau();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -240,6 +332,16 @@ double RAModel<SortPolicy>::Alpha() const
     return rStarTreeRA->Alpha();
   else if (xTreeRA)
     return xTreeRA->Alpha();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->Alpha();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->Alpha();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->Alpha();
+  else if (ubTreeRA)
+    return ubTreeRA->Alpha();
+  else if (octreeRA)
+    return octreeRA->Alpha();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -258,6 +360,16 @@ double& RAModel<SortPolicy>::Alpha()
     return rStarTreeRA->Alpha();
   else if (xTreeRA)
     return xTreeRA->Alpha();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->Alpha();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->Alpha();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->Alpha();
+  else if (ubTreeRA)
+    return ubTreeRA->Alpha();
+  else if (octreeRA)
+    return octreeRA->Alpha();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -276,6 +388,16 @@ bool RAModel<SortPolicy>::SampleAtLeaves() const
     return rStarTreeRA->SampleAtLeaves();
   else if (xTreeRA)
     return xTreeRA->SampleAtLeaves();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->SampleAtLeaves();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->SampleAtLeaves();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->SampleAtLeaves();
+  else if (ubTreeRA)
+    return ubTreeRA->SampleAtLeaves();
+  else if (octreeRA)
+    return octreeRA->SampleAtLeaves();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -294,6 +416,16 @@ bool& RAModel<SortPolicy>::SampleAtLeaves()
     return rStarTreeRA->SampleAtLeaves();
   else if (xTreeRA)
     return xTreeRA->SampleAtLeaves();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->SampleAtLeaves();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->SampleAtLeaves();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->SampleAtLeaves();
+  else if (ubTreeRA)
+    return ubTreeRA->SampleAtLeaves();
+  else if (octreeRA)
+    return octreeRA->SampleAtLeaves();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -312,6 +444,16 @@ bool RAModel<SortPolicy>::FirstLeafExact() const
     return rStarTreeRA->FirstLeafExact();
   else if (xTreeRA)
     return xTreeRA->FirstLeafExact();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->FirstLeafExact();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->FirstLeafExact();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->FirstLeafExact();
+  else if (ubTreeRA)
+    return ubTreeRA->FirstLeafExact();
+  else if (octreeRA)
+    return octreeRA->FirstLeafExact();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -330,6 +472,16 @@ bool& RAModel<SortPolicy>::FirstLeafExact()
     return rStarTreeRA->FirstLeafExact();
   else if (xTreeRA)
     return xTreeRA->FirstLeafExact();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->FirstLeafExact();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->FirstLeafExact();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->FirstLeafExact();
+  else if (ubTreeRA)
+    return ubTreeRA->FirstLeafExact();
+  else if (octreeRA)
+    return octreeRA->FirstLeafExact();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -348,6 +500,16 @@ size_t RAModel<SortPolicy>::SingleSampleLimit() const
     return rStarTreeRA->SingleSampleLimit();
   else if (xTreeRA)
     return xTreeRA->SingleSampleLimit();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->SingleSampleLimit();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->SingleSampleLimit();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->SingleSampleLimit();
+  else if (ubTreeRA)
+    return ubTreeRA->SingleSampleLimit();
+  else if (octreeRA)
+    return octreeRA->SingleSampleLimit();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -366,6 +528,16 @@ size_t& RAModel<SortPolicy>::SingleSampleLimit()
     return rStarTreeRA->SingleSampleLimit();
   else if (xTreeRA)
     return xTreeRA->SingleSampleLimit();
+  else if (hilbertRTreeRA)
+    return hilbertRTreeRA->SingleSampleLimit();
+  else if (rPlusTreeRA)
+    return rPlusTreeRA->SingleSampleLimit();
+  else if (rPlusPlusTreeRA)
+    return rPlusPlusTreeRA->SingleSampleLimit();
+  else if (ubTreeRA)
+    return ubTreeRA->SingleSampleLimit();
+  else if (octreeRA)
+    return octreeRA->SingleSampleLimit();
 
   throw std::runtime_error("no rank-approximate nearest neighbor search model "
       "initialized");
@@ -421,16 +593,18 @@ void RAModel<SortPolicy>::BuildModel(arma::mat&& referenceSet,
   }
 
   // Clean memory, if necessary.
-  if (kdTreeRA)
-    delete kdTreeRA;
-  if (coverTreeRA)
-    delete coverTreeRA;
-  if (rTreeRA)
-    delete rTreeRA;
-  if (rStarTreeRA)
-    delete rStarTreeRA;
-  if (xTreeRA)
-    delete xTreeRA;
+  delete kdTreeRA;
+  delete coverTreeRA;
+  delete rTreeRA;
+  delete rStarTreeRA;
+  delete xTreeRA;
+  delete hilbertRTreeRA;
+  delete rPlusTreeRA;
+  delete rPlusPlusTreeRA;
+  delete ubTreeRA;
+  delete octreeRA;
+
+  this->leafSize = leafSize;
 
   if (randomBasis)
     referenceSet = q * referenceSet;
@@ -479,6 +653,42 @@ void RAModel<SortPolicy>::BuildModel(arma::mat&& referenceSet,
       xTreeRA = new RAType<tree::XTree>(std::move(referenceSet), naive,
           singleMode);
       break;
+    case HILBERT_R_TREE:
+      hilbertRTreeRA = new RAType<tree::HilbertRTree>(std::move(referenceSet),
+          naive, singleMode);
+      break;
+    case R_PLUS_TREE:
+      rPlusTreeRA = new RAType<tree::RPlusTree>(std::move(referenceSet),
+          naive, singleMode);
+      break;
+    case R_PLUS_PLUS_TREE:
+      rPlusPlusTreeRA = new RAType<tree::RPlusPlusTree>(std::move(referenceSet),
+          naive, singleMode);
+      break;
+    case UB_TREE:
+      ubTreeRA = new RAType<tree::UBTree>(std::move(referenceSet),
+          naive, singleMode);
+      break;
+    case OCTREE:
+      // Build tree, if necessary.
+      if (naive)
+      {
+        octreeRA = new RAType<tree::Octree>(std::move(referenceSet), naive,
+            singleMode);
+      }
+      else
+      {
+        std::vector<size_t> oldFromNewReferences;
+        typename RAType<tree::Octree>::Tree* octree =
+            new typename RAType<tree::Octree>::Tree(std::move(referenceSet),
+            oldFromNewReferences, leafSize);
+        octreeRA = new RAType<tree::Octree>(octree, singleMode);
+
+        // Give the model ownership of the tree.
+        octreeRA->treeOwner = true;
+        octreeRA->oldFromNewReferences = oldFromNewReferences;
+      }
+      break;
   }
 
   if (!naive)
@@ -556,6 +766,53 @@ void RAModel<SortPolicy>::Search(arma::mat&& querySet,
       // No mapping necessary.
       xTreeRA->Search(querySet, k, neighbors, distances);
       break;
+    case HILBERT_R_TREE:
+      // No mapping necessary.
+      hilbertRTreeRA->Search(querySet, k, neighbors, distances);
+      break;
+    case R_PLUS_TREE:
+      // No mapping necessary.
+      rPlusTreeRA->Search(querySet, k, neighbors, distances);
+      break;
+    case R_PLUS_PLUS_TREE:
+      // No mapping necessary.
+      rPlusPlusTreeRA->Search(querySet, k, neighbors, distances);
+      break;
+    case UB_TREE:
+      // No mapping necessary.
+      ubTreeRA->Search(querySet, k, neighbors, distances);
+      break;
+    case OCTREE:
+      if (!octreeRA->Naive() && !octreeRA->SingleMode())
+      {
+        // Build a second tree and search.
+        Timer::Start("tree_building");
+        Log::Info << "Building query tree..." << std::endl;
+        std::vector<size_t> oldFromNewQueries;
+        typename RAType<tree::Octree>::Tree queryTree(std::move(querySet),
+            oldFromNewQueries, leafSize);
+        Log::Info << "Tree built." << std::endl;
+        Timer::Stop("tree_building");
+
+        arma::Mat<size_t> neighborsOut;
+        arma::mat distancesOut;
+        octreeRA->Search(&queryTree, k, neighborsOut, distancesOut);
+
+        // Unmap the query points.
+        distances.set_size(distancesOut.n_rows, distancesOut.n_cols);
+        neighbors.set_size(neighborsOut.n_rows, neighborsOut.n_cols);
+        for (size_t i = 0; i < neighborsOut.n_cols; ++i)
+        {
+          neighbors.col(oldFromNewQueries[i]) = neighborsOut.col(i);
+          distances.col(oldFromNewQueries[i]) = distancesOut.col(i);
+        }
+      }
+      else
+      {
+        // Search without building a second tree.
+        octreeRA->Search(querySet, k, neighbors, distances);
+      }
+      break;
   }
 }
 
@@ -590,6 +847,21 @@ void RAModel<SortPolicy>::Search(const size_t k,
     case X_TREE:
       xTreeRA->Search(k, neighbors, distances);
       break;
+    case HILBERT_R_TREE:
+      hilbertRTreeRA->Search(k, neighbors, distances);
+      break;
+    case R_PLUS_TREE:
+      rPlusTreeRA->Search(k, neighbors, distances);
+      break;
+    case R_PLUS_PLUS_TREE:
+      rPlusPlusTreeRA->Search(k, neighbors, distances);
+      break;
+    case UB_TREE:
+      ubTreeRA->Search(k, neighbors, distances);
+      break;
+    case OCTREE:
+      octreeRA->Search(k, neighbors, distances);
+      break;
   }
 }
 
@@ -608,6 +880,16 @@ std::string RAModel<SortPolicy>::TreeName() const
       return "R* tree";
     case X_TREE:
       return "X tree";
+    case HILBERT_R_TREE:
+      return "Hilbert R tree";
+    case R_PLUS_TREE:
+      return "R+ tree";
+    case R_PLUS_PLUS_TREE:
+      return "R++ tree";
+    case UB_TREE:
+      return "UB tree";
+    case OCTREE:
+      return "octree";
     default:
       return "unknown tree";
   }
