@@ -1,10 +1,7 @@
 /**
  * @file softmax_regression_main.cpp
- * @author Tham Ngap Wei
  *
- * Main executable for softmax regression.
- *
- * This file is part of mlpack 2.0.3.
+ * Main program for softmax regression.
  *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
@@ -48,32 +45,32 @@ PROGRAM_INFO("Softmax Regression", "This program performs softmax regression, "
     "on the given test set and its corresponding labels.");
 
 // Required options.
-PARAM_STRING("training_file", "A file containing the training set (the matrix "
-    "of predictors, X).", "t", "");
-PARAM_STRING("labels_file", "A file containing labels (0 or 1) for the points "
-    "in the training set (y). The labels must order as a row", "l", "");
+PARAM_STRING_IN("training_file", "A file containing the training set (the "
+    "matrix of predictors, X).", "t", "");
+PARAM_STRING_IN("labels_file", "A file containing labels (0 or 1) for the "
+    "points in the training set (y). The labels must order as a row.", "l", "");
 
 // Model loading/saving.
-PARAM_STRING("input_model_file", "File containing existing model (parameters).",
-    "m", "");
-PARAM_STRING("output_model_file", "File to save trained softmax regression "
-    "model to.", "M", "");
+PARAM_STRING_IN("input_model_file", "File containing existing model "
+    "(parameters).", "m", "");
+PARAM_STRING_OUT("output_model_file", "File to save trained softmax regression "
+    "model to.", "M");
 
 // Testing.
-PARAM_STRING("test_data", "File containing test dataset.", "T", "");
-PARAM_STRING("predictions_file", "File to save predictions for test dataset "
-    "into.", "p", "");
-PARAM_STRING("test_labels", "File containing test labels.", "L", "");
+PARAM_STRING_IN("test_data", "File containing test dataset.", "T", "");
+PARAM_STRING_OUT("predictions_file", "File to save predictions for test dataset"
+    " into.", "p");
+PARAM_STRING_IN("test_labels", "File containing test labels.", "L", "");
 
 // Softmax configuration options.
-PARAM_INT("max_iterations", "Maximum number of iterations before termination.",
-    "n", 400);
+PARAM_INT_IN("max_iterations", "Maximum number of iterations before "
+    "termination.", "n", 400);
 
-PARAM_INT("number_of_classes", "Number of classes for classification; if "
+PARAM_INT_IN("number_of_classes", "Number of classes for classification; if "
     "unspecified (or 0), the number of classes found in the labels will be "
     "used.", "c", 0);
 
-PARAM_DOUBLE("lambda", "L2-regularization constant", "r", 0.0001);
+PARAM_DOUBLE_IN("lambda", "L2-regularization constant", "r", 0.0001);
 
 PARAM_FLAG("no_intercept", "Do not add the intercept term to the model.", "N");
 
