@@ -3,8 +3,6 @@
  *
  * Test file for KNN class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -17,7 +15,7 @@
 #include <mlpack/core/tree/cover_tree.hpp>
 #include <mlpack/core/tree/example_tree.hpp>
 #include <boost/test/unit_test.hpp>
-#include "old_boost_test_definitions.hpp"
+#include "test_tools.hpp"
 
 using namespace mlpack;
 using namespace mlpack::neighbor;
@@ -185,7 +183,7 @@ BOOST_AUTO_TEST_CASE(EmptySearchTest)
       std::invalid_argument);
   BOOST_REQUIRE_THROW(empty.Search(5, neighbors, distances),
       std::invalid_argument);
-  BOOST_REQUIRE_THROW(empty.Search(&queryTree, 5, neighbors, distances),
+  BOOST_REQUIRE_THROW(empty.Search(queryTree, 5, neighbors, distances),
       std::invalid_argument);
 }
 
@@ -238,7 +236,7 @@ BOOST_AUTO_TEST_CASE(TrainTreeTest)
 
   std::vector<size_t> oldFromNewReferences;
   KNN::Tree tree(dataset, oldFromNewReferences);
-  empty.Train(&tree);
+  empty.Train(std::move(tree));
 
   empty.Search(5, neighbors, distances);
   baseline.Search(5, baselineNeighbors, baselineDistances);
@@ -277,12 +275,12 @@ BOOST_AUTO_TEST_CASE(TrainTreeTest)
  */
 BOOST_AUTO_TEST_CASE(NaiveTrainTreeTest)
 {
-  KNN empty(true);
+  KNN empty(NAIVE_MODE);
 
   arma::mat dataset = arma::randu<arma::mat>(5, 100);
   KNN::Tree tree(dataset);
 
-  BOOST_REQUIRE_THROW(empty.Train(&tree), std::invalid_argument);
+  BOOST_REQUIRE_THROW(empty.Train(std::move(tree)), std::invalid_argument);
 }
 
 /**
@@ -380,28 +378,32 @@ BOOST_AUTO_TEST_CASE(ExhaustiveSyntheticTest)
   // calculation.
   std::vector<size_t> oldFromNew;
   std::vector<size_t> newFromOld;
-  TreeType* tree = new TreeType(data, oldFromNew, newFromOld, 1);
+  TreeType tree(data, oldFromNew, newFromOld, 1);
+
+  KNN knn(std::move(tree));
+
   for (int i = 0; i < 3; i++)
   {
-    KNN* knn;
 
     switch (i)
     {
       case 0: // Use the dual-tree method.
-        knn = new KNN(tree, false);
+        knn.Naive() = false;
+        knn.SingleMode() = false;
         break;
       case 1: // Use the single-tree method.
-        knn = new KNN(tree, true);
+        knn.Naive() = false;
+        knn.SingleMode() = true;
         break;
       case 2: // Use the naive method.
-        knn = new KNN(tree->Dataset(), true);
+        knn.Naive() = true;
         break;
     }
 
     // Now perform the actual calculation.
     arma::Mat<size_t> neighbors;
     arma::mat distances;
-    knn->Search(10, neighbors, distances);
+    knn.Search(10, neighbors, distances);
 
     // Now the exhaustive check for correctness.  This will be long.  We must
     // also remember that the distances returned are squared distances.  As a
@@ -650,12 +652,7 @@ BOOST_AUTO_TEST_CASE(ExhaustiveSyntheticTest)
     BOOST_REQUIRE_EQUAL(neighbors(9, newFromOld[10]), newFromOld[4]);
     BOOST_REQUIRE_CLOSE(distances(9, newFromOld[10]), 4.05, 1e-5);
 
-    // Clean the memory.
-    delete knn;
   }
-
-  // Delete the tree.
-  delete tree;
 }
 
 /**
@@ -674,7 +671,7 @@ BOOST_AUTO_TEST_CASE(DualTreeVsNaive1)
 
   KNN knn(dataset);
 
-  KNN naive(dataset, true);
+  KNN naive(dataset, NAIVE_MODE);
 
   arma::Mat<size_t> neighborsTree;
   arma::mat distancesTree;
@@ -709,7 +706,7 @@ BOOST_AUTO_TEST_CASE(DualTreeVsNaive2)
   KNN knn(dataset);
 
   // Set naive mode.
-  KNN naive(dataset, true);
+  KNN naive(dataset, NAIVE_MODE);
 
   arma::Mat<size_t> neighborsTree;
   arma::mat distancesTree;
@@ -741,10 +738,10 @@ BOOST_AUTO_TEST_CASE(SingleTreeVsNaive)
   if (!data::Load("test_data_3_1000.csv", dataset))
     BOOST_FAIL("Cannot load test dataset test_data_3_1000.csv!");
 
-  KNN knn(dataset, false, true);
+  KNN knn(dataset, SINGLE_TREE_MODE);
 
   // Set up computation for naive mode.
-  KNN naive(dataset, true);
+  KNN naive(dataset, NAIVE_MODE);
 
   arma::Mat<size_t> neighborsTree;
   arma::mat distancesTree;
@@ -776,9 +773,9 @@ BOOST_AUTO_TEST_CASE(SingleCoverTreeTest)
       arma::mat> tree(data);
 
   NeighborSearch<NearestNeighborSort, LMetric<2>, arma::mat, StandardCoverTree>
-      coverTreeSearch(&tree, true);
+      coverTreeSearch(std::move(tree), SINGLE_TREE_MODE);
 
-  KNN naive(data, true);
+  KNN naive(data, NAIVE_MODE);
 
   arma::Mat<size_t> coverTreeNeighbors;
   arma::mat coverTreeDistances;
@@ -814,11 +811,11 @@ BOOST_AUTO_TEST_CASE(DualCoverTreeTest)
       arma::mat> referenceTree(dataset);
 
   NeighborSearch<NearestNeighborSort, EuclideanDistance, arma::mat,
-      StandardCoverTree> coverTreeSearch(&referenceTree);
+      StandardCoverTree> coverTreeSearch(std::move(referenceTree));
 
   arma::Mat<size_t> coverNeighbors;
   arma::mat coverDistances;
-  coverTreeSearch.Search(&referenceTree, 5, coverNeighbors, coverDistances);
+  coverTreeSearch.Search(dataset, 5, coverNeighbors, coverDistances);
 
   for (size_t i = 0; i < coverNeighbors.n_elem; ++i)
   {
@@ -842,14 +839,14 @@ BOOST_AUTO_TEST_CASE(SingleBallTreeTest)
       arma::mat> TreeType;
   TreeType tree(data);
 
+  KNN naive(tree.Dataset(), NAIVE_MODE);
+
   // BinarySpaceTree modifies data. Use modified data to maintain the
   // correspondance between points in the dataset for both methods. The order of
   // query points in both methods should be same.
 
   NeighborSearch<NearestNeighborSort, EuclideanDistance, arma::mat, BallTree>
-      ballTreeSearch(&tree, true);
-
-  KNN naive(tree.Dataset(), true);
+      ballTreeSearch(std::move(tree), SINGLE_TREE_MODE);
 
   arma::Mat<size_t> ballTreeNeighbors;
   arma::mat ballTreeDistances;
@@ -895,7 +892,99 @@ BOOST_AUTO_TEST_CASE(DualBallTreeTest)
   }
 }
 
-// Make sure sparse nearest neighbors works with kd trees.
+/**
+ * Test the spill tree hybrid sp-tree search (defeatist search on overlapping
+ * nodes, and backtracking in non-overlapping nodes) against the naive method.
+ * This uses only a random reference dataset.
+ */
+BOOST_AUTO_TEST_CASE(HybridSpillSearchTest)
+{
+  arma::mat dataset;
+  dataset.randu(50, 300); // 50 dimensional, 300 points.
+
+  const size_t k = 3;
+
+  KNN naive(dataset);
+  arma::Mat<size_t> neighborsNaive;
+  arma::mat distancesNaive;
+  naive.Search(dataset, k, neighborsNaive, distancesNaive);
+
+  double maxDist = 0;
+  for (size_t i = 0; i < neighborsNaive.n_cols; ++i)
+    if (distancesNaive(k - 1, i) > maxDist)
+      maxDist = distancesNaive(k - 1, i);
+
+  // If we are sure that tau is a valid strict upper bound of the kth nearest
+  // neighbor of the query points, then we can be sure that we will get an exact
+  // solution.
+  SpillKNN::Tree referenceTree(dataset, maxDist * 1.01 /* tau parameter */);
+  SpillKNN spTreeSearch(std::move(referenceTree));
+
+  for (size_t mode = 0; mode < 2; mode++)
+  {
+    spTreeSearch.SingleMode() = (mode == 0);
+
+    arma::Mat<size_t> neighborsSPTree;
+    arma::mat distancesSPTree;
+    spTreeSearch.Search(dataset, k, neighborsSPTree, distancesSPTree);
+
+    for (size_t i = 0; i < neighborsSPTree.n_elem; ++i)
+    {
+      BOOST_REQUIRE_EQUAL(neighborsSPTree(i), neighborsNaive(i));
+      BOOST_REQUIRE_CLOSE(distancesSPTree(i), distancesNaive(i), 1e-5);
+    }
+  }
+}
+
+/**
+ * Test hybrid sp-tree search doesn't repeat points.
+ * This uses only a random reference dataset.
+ */
+BOOST_AUTO_TEST_CASE(DuplicatedSpillSearchTest)
+{
+  arma::mat dataset;
+  dataset.randu(50, 300); // 50 dimensional, 300 points.
+
+  const size_t k = 15;
+
+  for (size_t test = 0; test < 2; test++)
+  {
+    double tau = test * 0.1;
+
+    SpillKNN::Tree referenceTree(dataset, tau);
+    SpillKNN spTreeSearch(std::move(referenceTree));
+
+    arma::Mat<size_t> neighborsSPTree;
+    arma::mat distancesSPTree;
+
+    for (size_t mode = 0; mode < 2; mode++)
+    {
+      spTreeSearch.SingleMode() = (mode == 0);
+
+      spTreeSearch.Search(dataset, k, neighborsSPTree, distancesSPTree);
+
+      for (size_t i = 0; i < neighborsSPTree.n_cols; ++i)
+      {
+        // Test that at least one point was found.
+        BOOST_REQUIRE(distancesSPTree(0, i) != DBL_MAX);
+
+        for (size_t j = 0; j < neighborsSPTree.n_rows; ++j)
+        {
+          if (distancesSPTree(j, i) == DBL_MAX)
+            break;
+          // All candidates with same distances must be different points.
+          for (size_t k = j + 1; k < neighborsSPTree.n_rows &&
+              distancesSPTree(k, i) == distancesSPTree(j, i); ++k)
+            BOOST_REQUIRE(neighborsSPTree(k, i) != neighborsSPTree(j, i));
+        }
+      }
+    }
+  }
+}
+
+/**
+ * Make sure sparse nearest neighbors works with kd trees.
+ */
 BOOST_AUTO_TEST_CASE(SparseKNNKDTreeTest)
 {
   // The dimensionality of these datasets must be high so that the probability
@@ -913,7 +1002,7 @@ BOOST_AUTO_TEST_CASE(SparseKNNKDTreeTest)
       KDTree> SparseKNN;
 
   SparseKNN a(referenceDataset);
-  KNN naive(denseReference, true);
+  KNN naive(denseReference, NAIVE_MODE);
 
   arma::mat sparseDistances;
   arma::Mat<size_t> sparseNeighbors;
@@ -982,7 +1071,7 @@ BOOST_AUTO_TEST_CASE(KNNModelTest)
   arma::mat referenceData = arma::randu<arma::mat>(10, 200);
 
   // Build all the possible models.
-  KNNModel models[12];
+  KNNModel models[28];
   models[0] = KNNModel(KNNModel::TreeTypes::KD_TREE, true);
   models[1] = KNNModel(KNNModel::TreeTypes::KD_TREE, false);
   models[2] = KNNModel(KNNModel::TreeTypes::COVER_TREE, true);
@@ -995,6 +1084,22 @@ BOOST_AUTO_TEST_CASE(KNNModelTest)
   models[9] = KNNModel(KNNModel::TreeTypes::X_TREE, false);
   models[10] = KNNModel(KNNModel::TreeTypes::BALL_TREE, true);
   models[11] = KNNModel(KNNModel::TreeTypes::BALL_TREE, false);
+  models[12] = KNNModel(KNNModel::TreeTypes::HILBERT_R_TREE, true);
+  models[13] = KNNModel(KNNModel::TreeTypes::HILBERT_R_TREE, false);
+  models[14] = KNNModel(KNNModel::TreeTypes::R_PLUS_TREE, true);
+  models[15] = KNNModel(KNNModel::TreeTypes::R_PLUS_TREE, false);
+  models[16] = KNNModel(KNNModel::TreeTypes::R_PLUS_PLUS_TREE, true);
+  models[17] = KNNModel(KNNModel::TreeTypes::R_PLUS_PLUS_TREE, false);
+  models[18] = KNNModel(KNNModel::TreeTypes::VP_TREE, true);
+  models[19] = KNNModel(KNNModel::TreeTypes::VP_TREE, false);
+  models[20] = KNNModel(KNNModel::TreeTypes::RP_TREE, true);
+  models[21] = KNNModel(KNNModel::TreeTypes::RP_TREE, false);
+  models[22] = KNNModel(KNNModel::TreeTypes::MAX_RP_TREE, true);
+  models[23] = KNNModel(KNNModel::TreeTypes::MAX_RP_TREE, false);
+  models[24] = KNNModel(KNNModel::TreeTypes::UB_TREE, true);
+  models[25] = KNNModel(KNNModel::TreeTypes::UB_TREE, false);
+  models[26] = KNNModel(KNNModel::TreeTypes::OCTREE, true);
+  models[27] = KNNModel(KNNModel::TreeTypes::OCTREE, false);
 
   for (size_t j = 0; j < 2; ++j)
   {
@@ -1004,17 +1109,18 @@ BOOST_AUTO_TEST_CASE(KNNModelTest)
     arma::mat baselineDistances;
     knn.Search(queryData, 3, baselineNeighbors, baselineDistances);
 
-    for (size_t i = 0; i < 12; ++i)
+    for (size_t i = 0; i < 28; ++i)
     {
       // We only have std::move() constructors so make a copy of our data.
       arma::mat referenceCopy(referenceData);
       arma::mat queryCopy(queryData);
       if (j == 0)
-        models[i].BuildModel(std::move(referenceCopy), 20, false, false);
+        models[i].BuildModel(std::move(referenceCopy), 20, DUAL_TREE_MODE);
       if (j == 1)
-        models[i].BuildModel(std::move(referenceCopy), 20, false, true);
+        models[i].BuildModel(std::move(referenceCopy), 20,
+            SINGLE_TREE_MODE);
       if (j == 2)
-        models[i].BuildModel(std::move(referenceCopy), 20, true, false);
+        models[i].BuildModel(std::move(referenceCopy), 20, NAIVE_MODE);
 
       arma::Mat<size_t> neighbors;
       arma::mat distances;
@@ -1048,7 +1154,7 @@ BOOST_AUTO_TEST_CASE(KNNModelMonochromaticTest)
   arma::mat referenceData = arma::randu<arma::mat>(10, 200);
 
   // Build all the possible models.
-  KNNModel models[12];
+  KNNModel models[28];
   models[0] = KNNModel(KNNModel::TreeTypes::KD_TREE, true);
   models[1] = KNNModel(KNNModel::TreeTypes::KD_TREE, false);
   models[2] = KNNModel(KNNModel::TreeTypes::COVER_TREE, true);
@@ -1061,6 +1167,22 @@ BOOST_AUTO_TEST_CASE(KNNModelMonochromaticTest)
   models[9] = KNNModel(KNNModel::TreeTypes::X_TREE, false);
   models[10] = KNNModel(KNNModel::TreeTypes::BALL_TREE, true);
   models[11] = KNNModel(KNNModel::TreeTypes::BALL_TREE, false);
+  models[12] = KNNModel(KNNModel::TreeTypes::HILBERT_R_TREE, true);
+  models[13] = KNNModel(KNNModel::TreeTypes::HILBERT_R_TREE, false);
+  models[14] = KNNModel(KNNModel::TreeTypes::R_PLUS_TREE, true);
+  models[15] = KNNModel(KNNModel::TreeTypes::R_PLUS_TREE, false);
+  models[16] = KNNModel(KNNModel::TreeTypes::R_PLUS_PLUS_TREE, true);
+  models[17] = KNNModel(KNNModel::TreeTypes::R_PLUS_PLUS_TREE, false);
+  models[18] = KNNModel(KNNModel::TreeTypes::VP_TREE, true);
+  models[19] = KNNModel(KNNModel::TreeTypes::VP_TREE, false);
+  models[20] = KNNModel(KNNModel::TreeTypes::RP_TREE, true);
+  models[21] = KNNModel(KNNModel::TreeTypes::RP_TREE, false);
+  models[22] = KNNModel(KNNModel::TreeTypes::MAX_RP_TREE, true);
+  models[23] = KNNModel(KNNModel::TreeTypes::MAX_RP_TREE, false);
+  models[24] = KNNModel(KNNModel::TreeTypes::UB_TREE, true);
+  models[25] = KNNModel(KNNModel::TreeTypes::UB_TREE, false);
+  models[26] = KNNModel(KNNModel::TreeTypes::OCTREE, true);
+  models[27] = KNNModel(KNNModel::TreeTypes::OCTREE, false);
 
   for (size_t j = 0; j < 2; ++j)
   {
@@ -1070,16 +1192,17 @@ BOOST_AUTO_TEST_CASE(KNNModelMonochromaticTest)
     arma::mat baselineDistances;
     knn.Search(3, baselineNeighbors, baselineDistances);
 
-    for (size_t i = 0; i < 12; ++i)
+    for (size_t i = 0; i < 28; ++i)
     {
       // We only have a std::move() constructor... so copy the data.
       arma::mat referenceCopy(referenceData);
       if (j == 0)
-        models[i].BuildModel(std::move(referenceCopy), 20, false, false);
+        models[i].BuildModel(std::move(referenceCopy), 20, DUAL_TREE_MODE);
       if (j == 1)
-        models[i].BuildModel(std::move(referenceCopy), 20, false, true);
+        models[i].BuildModel(std::move(referenceCopy), 20,
+            SINGLE_TREE_MODE);
       if (j == 2)
-        models[i].BuildModel(std::move(referenceCopy), 20, true, false);
+        models[i].BuildModel(std::move(referenceCopy), 20, NAIVE_MODE);
 
       arma::Mat<size_t> neighbors;
       arma::mat distances;
@@ -1137,7 +1260,7 @@ BOOST_AUTO_TEST_CASE(NeighborPtrDeleteTest)
   // Build the tree ourselves.
   std::vector<size_t> oldFromNewReferences;
   KNN::Tree tree(dataset);
-  KNN knn(&tree);
+  KNN knn(std::move(tree));
 
   // Now make a query set.
   arma::mat queryset = arma::randu<arma::mat>(5, 50);
