@@ -4,8 +4,6 @@
  * Unit tests for the 'RASearch' class and consequently the
  * 'RASearchRules' class
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -17,7 +15,7 @@
 #include <mlpack/core/tree/cover_tree.hpp>
 
 #include <boost/test/unit_test.hpp>
-#include "old_boost_test_definitions.hpp"
+#include "test_tools.hpp"
 
 #include <mlpack/methods/rann/ra_search.hpp>
 #include <mlpack/methods/rann/ra_model.hpp>
@@ -632,7 +630,7 @@ BOOST_AUTO_TEST_CASE(RAModelTest)
   data::Load("rann_test_q_3_100.csv", queryData, true);
 
   // Build all the possible models.
-  KNNModel models[10];
+  KNNModel models[20];
   models[0] = KNNModel(KNNModel::TreeTypes::KD_TREE, false);
   models[1] = KNNModel(KNNModel::TreeTypes::KD_TREE, true);
   models[2] = KNNModel(KNNModel::TreeTypes::COVER_TREE, false);
@@ -643,13 +641,23 @@ BOOST_AUTO_TEST_CASE(RAModelTest)
   models[7] = KNNModel(KNNModel::TreeTypes::R_STAR_TREE, true);
   models[8] = KNNModel(KNNModel::TreeTypes::X_TREE, false);
   models[9] = KNNModel(KNNModel::TreeTypes::X_TREE, true);
+  models[10] = KNNModel(KNNModel::TreeTypes::HILBERT_R_TREE, false);
+  models[11] = KNNModel(KNNModel::TreeTypes::HILBERT_R_TREE, true);
+  models[12] = KNNModel(KNNModel::TreeTypes::R_PLUS_TREE, false);
+  models[13] = KNNModel(KNNModel::TreeTypes::R_PLUS_TREE, true);
+  models[14] = KNNModel(KNNModel::TreeTypes::R_PLUS_PLUS_TREE, false);
+  models[15] = KNNModel(KNNModel::TreeTypes::R_PLUS_PLUS_TREE, true);
+  models[16] = KNNModel(KNNModel::TreeTypes::UB_TREE, false);
+  models[17] = KNNModel(KNNModel::TreeTypes::UB_TREE, true);
+  models[18] = KNNModel(KNNModel::TreeTypes::OCTREE, false);
+  models[19] = KNNModel(KNNModel::TreeTypes::OCTREE, true);
 
   arma::Mat<size_t> qrRanks;
   data::Load("rann_test_qr_ranks.csv", qrRanks, true, false); // No transpose.
 
   for (size_t j = 0; j < 3; ++j)
   {
-    for (size_t i = 0; i < 10; ++i)
+    for (size_t i = 0; i < 20; ++i)
     {
       // We only have std::move() constructors so make a copy of our data.
       arma::mat referenceCopy(referenceData);
@@ -700,7 +708,7 @@ BOOST_AUTO_TEST_CASE(RAModelTest)
 
       // assert that at most 5% of the queries fall out of this threshold
       // 5% of 100 queries is 5.
-      size_t maxNumQueriesFail = 12; // Looser bound due to multiple trials.
+      size_t maxNumQueriesFail = 25; // See #734 for why this is so high.
 
       BOOST_REQUIRE_LT(numQueriesFail, maxNumQueriesFail);
     }
