@@ -4,8 +4,6 @@
  *
  * Tests for data::Load() and data::Save().
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -16,7 +14,7 @@
 #include <mlpack/core.hpp>
 
 #include <boost/test/unit_test.hpp>
-#include "old_boost_test_definitions.hpp"
+#include "test_tools.hpp"
 
 using namespace mlpack;
 using namespace mlpack::data;
@@ -525,9 +523,13 @@ BOOST_AUTO_TEST_CASE(SavePGMBinaryTest)
   remove("test_file.pgm");
 }
 
-#if defined(ARMA_USE_HDF5) && (ARMA_VERSION_MAJOR == 3 \
-    || (ARMA_VERSION_MAJOR == 4 && (ARMA_VERSION_MINOR < 300 \
-    ||  ARMA_VERSION_MINOR > 400)))
+// Don't perform any HDF5 tests on Armadillo 4.300-4.400 (inclusive).  A bug
+// causes loading to fail.
+#if ((ARMA_VERSION_MAJOR == 4) && \
+        (ARMA_VERSION_MINOR < 300 || ARMA_VERSION_MINOR > 400)) || \
+    (ARMA_VERSION_MAJOR >= 5)
+
+#if defined(ARMA_USE_HDF5)
 /**
  * Make sure load as HDF5 is successful.
  */
@@ -659,6 +661,8 @@ BOOST_AUTO_TEST_CASE(NoHDF5Test)
 }
 #endif
 
+#endif
+
 /**
  * Test normalization of labels.
  */
@@ -1408,6 +1412,8 @@ BOOST_AUTO_TEST_CASE(HarderKeonTest)
   BOOST_REQUIRE_EQUAL(ntInfo.NumMappings(1), 5);
   BOOST_REQUIRE_EQUAL(ntInfo.NumMappings(2), 5);
   BOOST_REQUIRE_EQUAL(ntInfo.NumMappings(3), 3);
+
+  remove("test.csv");
 }
 
 /**
