@@ -5,8 +5,6 @@
  * global test fixture for each test. Each individual test is contained in
  * its own file.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -24,7 +22,7 @@
 #endif
 
 #include <boost/test/unit_test.hpp>
-#include "old_boost_test_definitions.hpp"
+#include "test_tools.hpp"
 
 /**
  * Provide a global fixture for each test.
