@@ -4,8 +4,6 @@
  *
  * Test file for RangeSearch<> class.
  *
- * This file is part of mlpack 2.0.3.
- *
  * mlpack is free software; you may redistribute it and/or modify it under the
  * terms of the 3-clause BSD license.  You should have received a copy of the
  * 3-clause BSD license along with mlpack.  If not, see
@@ -16,7 +14,7 @@
 #include <mlpack/core/tree/cover_tree.hpp>
 #include <mlpack/methods/range_search/rs_model.hpp>
 #include <boost/test/unit_test.hpp>
-#include "old_boost_test_definitions.hpp"
+#include "test_tools.hpp"
 
 using namespace mlpack;
 using namespace mlpack::range;
@@ -1251,14 +1249,12 @@ BOOST_AUTO_TEST_CASE(MoveTrainTest)
 
 BOOST_AUTO_TEST_CASE(RSModelTest)
 {
-  math::RandomSeed(std::time(NULL));
-
   // Ensure that we can build an RSModel and get correct results.
   arma::mat queryData = arma::randu<arma::mat>(10, 50);
   arma::mat referenceData = arma::randu<arma::mat>(10, 200);
 
   // Build all the possible models.
-  RSModel models[12];
+  RSModel models[28];
   models[0] = RSModel(RSModel::TreeTypes::KD_TREE, true);
   models[1] = RSModel(RSModel::TreeTypes::KD_TREE, false);
   models[2] = RSModel(RSModel::TreeTypes::COVER_TREE, true);
@@ -1271,6 +1267,22 @@ BOOST_AUTO_TEST_CASE(RSModelTest)
   models[9] = RSModel(RSModel::TreeTypes::X_TREE, false);
   models[10] = RSModel(RSModel::TreeTypes::BALL_TREE, true);
   models[11] = RSModel(RSModel::TreeTypes::BALL_TREE, false);
+  models[12] = RSModel(RSModel::TreeTypes::HILBERT_R_TREE, true);
+  models[13] = RSModel(RSModel::TreeTypes::HILBERT_R_TREE, false);
+  models[14] = RSModel(RSModel::TreeTypes::R_PLUS_TREE, true);
+  models[15] = RSModel(RSModel::TreeTypes::R_PLUS_TREE, false);
+  models[16] = RSModel(RSModel::TreeTypes::R_PLUS_PLUS_TREE, true);
+  models[17] = RSModel(RSModel::TreeTypes::R_PLUS_PLUS_TREE, false);
+  models[18] = RSModel(RSModel::TreeTypes::VP_TREE, true);
+  models[19] = RSModel(RSModel::TreeTypes::VP_TREE, false);
+  models[20] = RSModel(RSModel::TreeTypes::RP_TREE, true);
+  models[21] = RSModel(RSModel::TreeTypes::RP_TREE, false);
+  models[22] = RSModel(RSModel::TreeTypes::MAX_RP_TREE, true);
+  models[23] = RSModel(RSModel::TreeTypes::MAX_RP_TREE, false);
+  models[24] = RSModel(RSModel::TreeTypes::UB_TREE, true);
+  models[25] = RSModel(RSModel::TreeTypes::UB_TREE, false);
+  models[26] = RSModel(RSModel::TreeTypes::OCTREE, true);
+  models[27] = RSModel(RSModel::TreeTypes::OCTREE, false);
 
   for (size_t j = 0; j < 2; ++j)
   {
@@ -1284,7 +1296,7 @@ BOOST_AUTO_TEST_CASE(RSModelTest)
     vector<vector<pair<double, size_t>>> baselineSorted;
     SortResults(baselineNeighbors, baselineDistances, baselineSorted);
 
-    for (size_t i = 0; i < 12; ++i)
+    for (size_t i = 0; i < 28; ++i)
     {
       // We only have std::move() constructors, so make a copy of our data.
       arma::mat referenceCopy(referenceData);
@@ -1328,7 +1340,7 @@ BOOST_AUTO_TEST_CASE(RSModelMonochromaticTest)
   arma::mat referenceData = arma::randu<arma::mat>(10, 200);
 
   // Build all the possible models.
-  RSModel models[12];
+  RSModel models[28];
   models[0] = RSModel(RSModel::TreeTypes::KD_TREE, true);
   models[1] = RSModel(RSModel::TreeTypes::KD_TREE, false);
   models[2] = RSModel(RSModel::TreeTypes::COVER_TREE, true);
@@ -1341,6 +1353,22 @@ BOOST_AUTO_TEST_CASE(RSModelMonochromaticTest)
   models[9] = RSModel(RSModel::TreeTypes::X_TREE, false);
   models[10] = RSModel(RSModel::TreeTypes::BALL_TREE, true);
   models[11] = RSModel(RSModel::TreeTypes::BALL_TREE, false);
+  models[12] = RSModel(RSModel::TreeTypes::HILBERT_R_TREE, true);
+  models[13] = RSModel(RSModel::TreeTypes::HILBERT_R_TREE, false);
+  models[14] = RSModel(RSModel::TreeTypes::R_PLUS_TREE, true);
+  models[15] = RSModel(RSModel::TreeTypes::R_PLUS_TREE, false);
+  models[16] = RSModel(RSModel::TreeTypes::R_PLUS_PLUS_TREE, true);
+  models[17] = RSModel(RSModel::TreeTypes::R_PLUS_PLUS_TREE, false);
+  models[18] = RSModel(RSModel::TreeTypes::VP_TREE, true);
+  models[19] = RSModel(RSModel::TreeTypes::VP_TREE, false);
+  models[20] = RSModel(RSModel::TreeTypes::RP_TREE, true);
+  models[21] = RSModel(RSModel::TreeTypes::RP_TREE, false);
+  models[22] = RSModel(RSModel::TreeTypes::MAX_RP_TREE, true);
+  models[23] = RSModel(RSModel::TreeTypes::MAX_RP_TREE, false);
+  models[24] = RSModel(RSModel::TreeTypes::MAX_RP_TREE, true);
+  models[25] = RSModel(RSModel::TreeTypes::MAX_RP_TREE, false);
+  models[26] = RSModel(RSModel::TreeTypes::OCTREE, true);
+  models[27] = RSModel(RSModel::TreeTypes::OCTREE, false);
 
   for (size_t j = 0; j < 2; ++j)
   {
@@ -1353,7 +1381,7 @@ BOOST_AUTO_TEST_CASE(RSModelMonochromaticTest)
     vector<vector<pair<double, size_t>>> baselineSorted;
     SortResults(baselineNeighbors, baselineDistances, baselineSorted);
 
-    for (size_t i = 0; i < 12; ++i)
+    for (size_t i = 0; i < 28; ++i)
     {
       // We only have std::move() cosntructors, so make a copy of our data.
       arma::mat referenceCopy(referenceData);
